/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __CLIENT__
#define __CLIENT__

#define	SIGNONS			4

#define	MAX_MAPSTRING	2048
#define	MAX_DEMOS		8
#define	MAX_DEMONAME	16

typedef struct
{
	float	laprecord;
	float	racerecord;
	char	name[32];
	char	description[32];
	char	lapholder[32];
	char	raceholder[32];
} map_t;

typedef struct
{
	char		name[32];
	float		totaltime;
	float		fastestlap;
	byte		on_lap;
	qboolean	laprecord;
	qboolean	racerecord;
} times_t;

typedef struct countdown_s
{
	float		time;
	qboolean	used;
} countdown_t;

countdown_t countdown;

typedef enum 
{
	ca_dedicated,
	ca_disconnected,
	ca_connected
} cactive_t;

typedef struct
{
	struct qsocket_s	*netcon;
	cactive_t			state;
	sizebuf_t			message;

	float				td_starttime;
	int					td_lastframe;
	int					td_startframe;
	int					demonum;
	int					signon;

	char				mapstring[MAX_QPATH];
	char				demos[MAX_DEMOS][MAX_DEMONAME];

	qboolean			demorecording;
	qboolean			demoplayback;
	qboolean			timedemo;
	
	FILE				*demofile;
} client_static_t;

extern client_static_t	cls;

typedef struct
{
	struct model_s	*model_precache[MAX_MODELS];
	struct model_s	*worldmodel;
	struct model_s	*skinmodel[8];

	times_t			*times;

	double			mtime[2];
	double			time;
	double			oldtime;

	float			last_received_message;

	int				movemessages;
	int				viewentity;
	int				maxclients;
	int				num_entities;

	byte			carcolor[8][3];
	byte			skincarcolor[8][3];
} client_state_t;

//
// cvars
//
extern	cvar_t	_cl_name;
extern	cvar_t	_cl_color_red;
extern	cvar_t	_cl_color_green;
extern	cvar_t	_cl_color_blue;
extern	cvar_t	_cl_shownet;

extern	client_state_t	cl;

extern	entity_t		cl_entities[MAX_EDICTS];

//=============================================================================

//
// cl_main
//

void CL_Init (void);

void CL_EstablishConnection (char *host);
void CL_Signon1 (void);
void CL_Signon2 (void);
void CL_Signon3 (void);
void CL_Signon4 (void);
void CL_SignonReply (void);
void CL_Disconnect (void);
void CL_Disconnect_f (void);
void CL_NextDemo (void);

#define	MAX_VISEDICTS_CAR	8
#define	MAX_VISEDICTS_BRUSH	256

extern	int			cl_numvisedicts_car;
extern	int			cl_numvisedicts_brush;
extern	entity_t	*cl_visedicts_car[MAX_VISEDICTS_CAR];
extern	entity_t	*cl_visedicts_brush[MAX_VISEDICTS_BRUSH];

//
// cl_input
//
typedef struct
{
	int		down[2];		// key nums holding it down
	int		state;			// low bit is down state
} kbutton_t;

int  CL_ReadFromServer (void);
void CL_InitInput (void);
void CL_SendCmd (void);
void CL_SendMove ();
void CL_ClearState (void);
void CL_WriteToServer ();
char *Key_KeynumToString (int keynum);

//
// cl_demo.c
//
int	 CL_GetMessage (void);
void CL_StopPlayback (void);

void CL_Stop_f (void);
void CL_Record_f (void);
void CL_PlayDemo_f (void);
void CL_TimeDemo_f (void);

//
// cl_parse.c
//
void CL_ParseServerMessage (void);
void CL_NewTranslation (int slot);

#endif //__CLIENT__
