/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// cl_parse.c  -- parse a message received from the server

#include	"../quakedef.h"

extern	map_t	maps[3];
extern	int		startmap;

qboolean		showtimes;

char *svc_strings[] =
{
	"svc_bad",					// 0
	"svc_nop",					// 1
	"svc_disconnect",			// 2
	"svc_setview",				// 3
	"svc_time",					// 4
	"svc_print",				// 5
	"svc_stufftext",			// 6
	"svc_serverinfo",			// 7
	"svc_updatename",			// 8
	"svc_signonnum",			// 9
	"svc_updatecolors",			// 10
	"svc_updatetime",			// 11
	"svc_countdown",			// 12
	"svc_raceover",				// 13
	"svc_updatelaps",			// 14
	"svc_collision",			// 15
	"svc_spawnbaseline",		// 16
};

//=============================================================================

/*
============
CL_EntityNum
============
*/
entity_t *CL_EntityNum (int num)
{
	if (num >= cl.num_entities)
	{
		if (num >= MAX_EDICTS)
			Host_Error ("CL_EntityNum: %i is an invalid number",num);
		while (cl.num_entities<=num)
		{
			cl.num_entities++;
		}
	}
		
	return &cl_entities[num];
}

/*
===================
CL_KeepaliveMessage
===================
*/
void CL_KeepaliveMessage (void)
{
	float			time;
	static	float	lastmsg;
	int				ret;
	sizebuf_t		old;
	byte			olddata[8192];
	
	if (sv.active || cls.demoplayback)
		return;

	old = net_message;
	MR_memcpy (olddata, net_message.data, net_message.cursize);
	
	do
	{
		ret = CL_GetMessage ();
		switch (ret)
		{
		default:
			Host_Error ("CL_KeepaliveMessage: CL_GetMessage failed");		
		case 0:
			break;
		case 1:
			Host_Error ("CL_KeepaliveMessage: received a message");
			break;
		case 2:
			if (MSG_ReadByte() != svc_nop)
				Host_Error ("CL_KeepaliveMessage: datagram wasn't a nop");
			break;
		}
	} while (ret);

	net_message = old;
	MR_memcpy (net_message.data, olddata, net_message.cursize);

	time = Sys_FloatTime ();
	if (time - lastmsg < 5)
		return;
	lastmsg = time;

	Con_Printf ("--> client to server keepalive\n");

	MSG_WriteByte (&cls.message, clc_nop);
	NET_SendMessage (cls.netcon, &cls.message);
	SZ_Clear (&cls.message);
}

/*
==================
CL_ParseServerInfo
==================
*/
void CL_ParseServerInfo (void)
{
	char	*str;
	int		i;
	int		nummodels;
	char	model_precache[MAX_MODELS][MAX_QPATH];
	
	Con_DPrintf ("Serverinfo packet received.\n");

	CL_ClearState ();

	cl.maxclients = MSG_ReadByte ();
	if (cl.maxclients < 1 || cl.maxclients > 8)
	{
		Con_Printf("Bad maxclients (%u) from server\n", cl.maxclients);
		return;
	}
	cl.times = Hunk_AllocName (cl.maxclients*sizeof(*cl.times), "times");

	str = MSG_ReadString ();

	Con_Printf("\n\35\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\37\n");
	Con_Printf ("%c%s", 2, str);
	Con_Printf("\n\35\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\37\n");

	MR_memset (cl.model_precache, 0, sizeof(cl.model_precache));
	for (nummodels=1 ; ; nummodels++)
	{
		str = MSG_ReadString ();
		if (!str[0])
			break;
		if (nummodels==MAX_MODELS)
		{
			Con_Printf ("Server sent too many model precaches\n");
			return;
		}
		MR_strcpy (model_precache[nummodels], str);
	}

	for (i=1 ; i<nummodels ; i++)
	{
		cl.model_precache[i] = Mod_ForName (model_precache[i], false);
		if (cl.model_precache[i] == NULL)
		{
			Con_Printf("Model %s not found\n", model_precache[i]);
			return;
		}
		CL_KeepaliveMessage ();
	}

	cl_entities[0].model = cl.worldmodel = cl.model_precache[1];
	
	R_NewMap ();
}

/*
==============
CL_ParseUpdate
==============
*/
void ColorSkin(int carnum);

void CL_ParseUpdate (int bits)
{
	model_t		*model;
	qboolean	forcelink;
	entity_t	*ent;
	int			num, modnum;

	if (cls.signon == SIGNONS - 1)
	{
		cls.signon = SIGNONS;
		CL_SignonReply ();
	}

	num = MSG_ReadByte ();

	ent = CL_EntityNum (num);

	if (ent->msgtime != cl.mtime[1])
		forcelink = true;
	else
		forcelink = false;

	ent->msgtime = cl.mtime[0];

	if (bits & U_MODEL)
	{
		modnum = MSG_ReadByte ();
		if (modnum >= MAX_MODELS)
			Host_Error ("CL_ParseModel: bad modnum");
	}
	else
		modnum = ent->baseline.modelindex;
		
	model = cl.model_precache[modnum];

	if (model != ent->model)
	{
		ent->model = model;

		if (!model)
			forcelink = true;
	}
	
	VectorCopy (ent->msg_angles[0], ent->msg_angles[1]);
	VectorCopy (ent->msg_origin[0], ent->msg_origin[1]);

	if (bits & U_ANGLE0)		ent->msg_angles[0][0]	= MSG_ReadAngle();
	else						ent->msg_angles[0][0]	= ent->baseline.angles[0];
	if (bits & U_ANGLE1)		ent->msg_angles[0][1]	= MSG_ReadAngle();
	else						ent->msg_angles[0][1]	= ent->baseline.angles[1];
	if (bits & U_ANGLE2)		ent->msg_angles[0][2]	= MSG_ReadAngle();
	else						ent->msg_angles[0][2]	= ent->baseline.angles[2];
	if (bits & U_ORIGIN0)		ent->msg_origin[0][0]	= MSG_ReadCoord ();
	else						ent->msg_origin[0][0]	= ent->baseline.origin[0];
	if (bits & U_ORIGIN1)		ent->msg_origin[0][1]	= MSG_ReadCoord ();
	else						ent->msg_origin[0][1]	= ent->baseline.origin[1];	
	if (bits & U_ORIGIN2)		ent->msg_origin[0][2]	= MSG_ReadCoord ();
	else						ent->msg_origin[0][2]	= ent->baseline.origin[2];

	if (forcelink)
	{
		VectorCopy (ent->msg_angles[0], ent->angles);
		VectorCopy (ent->msg_origin[0], ent->origin);
		ent->forcelink = true;
	}
}

/*
================
CL_ParseBaseline
================
*/
void CL_ParseBaseline (entity_t *ent)
{
	ent->baseline.modelindex	= MSG_ReadByte ();
	ent->baseline.angles[0]		= MSG_ReadAngle ();
	ent->baseline.angles[1]		= MSG_ReadAngle ();
	ent->baseline.angles[2]		= MSG_ReadAngle ();
	ent->baseline.origin[0]		= MSG_ReadCoord ();
	ent->baseline.origin[1]		= MSG_ReadCoord ();
	ent->baseline.origin[2]		= MSG_ReadCoord ();
}

/*
=====================
CL_ParseServerMessage
=====================
*/
void CL_ParseServerMessage (void)
{
	vec3_t		origin;
	int			cmd;
	int			i;
	int			loop = 1;

	if (_cl_shownet.ivalue)
		Con_Printf ("%i ",net_message.cursize);

	MSG_BeginReading ();
	
	while (loop)
	{
		if (msg_badread)
			Host_Error ("CL_ParseServerMessage: Bad server message");

		cmd = MSG_ReadByte ();

		if (cmd == -1)
			return;

		if (cmd & 128)
		{
			CL_ParseUpdate (cmd&127);
			continue;
		}

		switch (cmd)
		{
		default:
			Con_Printf ("%3i:%s\n", msg_readcount-1, svc_strings[cmd]);
			Host_Error ("CL_ParseServerMessage: Illegible server message\n");
			break;
			
		case svc_nop:
			break;
			
		case svc_time:
			cl.mtime[1] = cl.mtime[0];
			cl.mtime[0] = MSG_ReadFloat ();			
			break;
					
		case svc_disconnect:
			Host_EndGame ("Server disconnected\n");

		case svc_print:
			Con_Printf ("%s", MSG_ReadString ());
			break;
			
		case svc_stufftext:
			Cbuf_AddText (MSG_ReadString ());
			break;
						
		case svc_serverinfo:
			CL_ParseServerInfo ();
			showtimes = false;
			break;
						
		case svc_setview:
			cl.viewentity = MSG_ReadShort ();
			break;
					
		case svc_updatename:
			i = MSG_ReadByte ();
			if (i >= cl.maxclients)
				Host_Error ("CL_ParseServerMessage: svc_updatename > 8");
			MR_strcpy (cl.times[i].name, MSG_ReadString ());
			break;

		case svc_signonnum:
			i = MSG_ReadByte ();
			if (i <= cls.signon)
				Host_Error ("Received signon %i when at %i", i, cls.signon);
			cls.signon = i;
			CL_SignonReply ();
			break;

		case svc_updatecolors:
			i = MSG_ReadByte ();
			if (i >= cl.maxclients)
				Host_Error ("CL_ParseServerMessage: svc_updatecolors > 8");
			cl.carcolor[i][0] = MSG_ReadByte ();
			cl.carcolor[i][1] = MSG_ReadByte ();
			cl.carcolor[i][2] = MSG_ReadByte ();
			ColorSkin (i);
			break;

		case svc_updatetime:
			i = MSG_ReadByte ();
			if (i >= cl.maxclients)
				Host_Error ("CL_ParseServerMessage: svc_updatetime > 8");
			cl.times[i].totaltime	= MSG_ReadFloat ();
			cl.times[i].fastestlap	= MSG_ReadFloat ();
			break;

		case svc_countdown:
			countdown.used = true;
			countdown.time = cl.time;
			break;

		case svc_raceover:
			i = MSG_ReadByte ();
			if (i >= cl.maxclients)
				Host_Error ("CL_ParseServerMessage: svc_raceover > 8");
			if (i == (cl.viewentity - 1))
				showtimes = true;

			if (cl.times[i].fastestlap < maps[startmap].laprecord)
			{
				maps[startmap].laprecord = cl.times[i].fastestlap;
				MR_strcpy (maps[startmap].lapholder, cl.times[i].name);
				cl.times[i].laprecord = true;
			}

			if (cl.times[i].totaltime < maps[startmap].racerecord)
			{
				maps[startmap].racerecord = cl.times[i].totaltime;
				MR_strcpy (maps[startmap].raceholder, cl.times[i].name);
				cl.times[i].racerecord = true;
			}
			break;

		case svc_updatelaps:
			i = MSG_ReadByte ();
			if (i >= cl.maxclients)
				Host_Error ("CL_ParseServerMessage: svc_updatelaps > 8");
			cl.times[i].on_lap = MSG_ReadByte ();
			break;

		case svc_collision:
			origin[0] = MSG_ReadCoord ();
			origin[1] = MSG_ReadCoord ();
			origin[2] = MSG_ReadCoord ();
			break;

		case svc_spawnbaseline:
			i = MSG_ReadShort ();
			CL_ParseBaseline (CL_EntityNum(i));
			break;
		}
	}
}
