/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"
#include	"../game.h"

void search_for_way (edict_t *ent)
{
	if (ent->game.havegoal)
		return;

	ent->game.way_timer = sv.time;

	VectorCopy (miniracer.waypoint[ent->game.ways].origin, ent->game.goalorigin);

	ent->game.havegoal = true;
}

void are_we_there_yet (edict_t *ent)
{
	vec3_t	origin;

	if (!ent->game.havegoal)
		return;

	if (sv.time - ent->game.way_timer > 10 && ent->game.ways != 0 && sv.time - miniracer.tele_timer > 1)
	{
		ent->game.origin[0] = miniracer.waypoint[ent->game.ways - 1].origin[0];
		ent->game.origin[1] = miniracer.waypoint[ent->game.ways - 1].origin[1];
		ent->game.origin[2] = miniracer.waypoint[ent->game.ways - 1].origin[2] + 16;
		VectorClear (ent->game.velocity);
		miniracer.tele_timer = ent->game.way_timer = sv.time;
		return;
	}

	VectorSubtract (ent->game.origin, ent->game.goalorigin, origin);

	if (Length (origin) < 100)
	{
		ent->game.havegoal	= false;
		ent->game.ways		= ent->game.ways + 1;

		if (ent->game.ways == miniracer.TotalWays)
			ent->game.ways = 0;
	}
}

void MoveBot (edict_t *ent)
{
	vec3_t	x, x1, x2, z, z1, z2, v;
	vec3_t	origin;
	vec3_t	origin2;

	if (ent->game.finished)
		return;

	if (ent->game.laptime[3])
	{
		ent->game.finished = true;
		RotateCar ();
		return;
	}

	if(ent->game.ready)
	{
		trace_t	trace;

		search_for_way (ent);
		are_we_there_yet (ent);

		origin[0] = ent->game.origin[0];
		origin[1] = ent->game.origin[1];
		origin[2] = ent->game.origin[2] - 16;

		trace = Traceline (ent->game.origin, origin, ent);

		if (ent->game.havegoal && trace.fraction != 1)
		{
			vec3_t	forward, right, up;

			origin[0] = ent->game.goalorigin[0] - ent->game.origin[0];
			origin[1] = ent->game.goalorigin[1] - ent->game.origin[1];
			origin[2] = ent->game.goalorigin[2] - ent->game.origin[2];

			ent->game.ideal_yaw	= VectoYaw(origin);

			CalcYaw (ent);

			ChangeAngles (ent);
			
			AngleVectors (ent->v.angles, forward, right, up);

			ent->game.velocity[0] += ((forward[0] * ent->game.bot_speed) - ent->game.velocity[0]) * host_frametime;
			ent->game.velocity[1] += ((forward[1] * ent->game.bot_speed) - ent->game.velocity[1]) * host_frametime;
			ent->game.velocity[2] += ((forward[2] * ent->game.bot_speed) - ent->game.velocity[2]) * host_frametime;

			ent->game.velocity[0] -= ent->game.velocity[0] * host_frametime * 0.45;
			ent->game.velocity[1] -= ent->game.velocity[1] * host_frametime * 0.45;
			ent->game.velocity[2] -= ent->game.velocity[2] * host_frametime * 0.45;

			forward[0]	*= 12;
			forward[1]	*= 12;
			forward[2]	*= 12;

			right[0]	*= 6;
			right[1]	*= 6;
			right[2]	*= 6;

			// first see which way the ground slopes in front of you
			origin[0] = ent->game.origin[0] + forward[0];
			origin[1] = ent->game.origin[1] + forward[1];
			origin[2] = ent->game.origin[2] + forward[2];

			origin2[0] = origin[0];
			origin2[1] = origin[1];
			origin2[2] = origin[2] - 16;

			trace = Traceline (origin, origin2, ent);
			if (trace.fraction == 1)
				return;

			VectorCopy (trace.endpos, x1);

			// second see which way the ground slopes behind you
			origin[0] = ent->game.origin[0] - forward[0];
			origin[1] = ent->game.origin[1] - forward[1];
			origin[2] = ent->game.origin[2] - forward[2];

			origin2[0] = origin[0];
			origin2[1] = origin[1];
			origin2[2] = origin[2] - 16;

			trace = Traceline (origin, origin2, ent);
			if (trace.fraction == 1)
				return;

			VectorCopy (trace.endpos, x2);

			// third see which way the ground slopes to the left
			origin[0] = ent->game.origin[0] - right[0];
			origin[1] = ent->game.origin[1] - right[1];
			origin[2] = ent->game.origin[2] - right[2];

			origin2[0] = origin[0];
			origin2[1] = origin[1];
			origin2[2] = origin[2] - 16;

			trace = Traceline (origin, origin2, ent);
			if (trace.fraction == 1)
				return;

			VectorCopy (trace.endpos, z1);
	
			// finally see which way the ground slopes to the right
			origin[0] = ent->game.origin[0] + right[0];
			origin[1] = ent->game.origin[1] + right[1];
			origin[2] = ent->game.origin[2] + right[2];

			origin2[0] = origin[0];
			origin2[1] = origin[1];
			origin2[2] = origin[2] - 16;

			trace = Traceline (origin, origin2, ent);
			if (trace.fraction == 1)
				return;

			VectorCopy (trace.endpos, z2);

			VectorSubtract (x1, x2 ,x);
			VectorSubtract (z1, z2 ,z);

			Normalize (x, v);
			VectoAngles (v, v);
			ent->v.angles[0] = v[0];
		
			Normalize (z, v);
			VectoAngles (v, v);
			ent->v.angles[2] = v[0];
		}
	}
}
