/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"
#include	"../game.h"

void WorldSpawn (void)
{
	int	i;

	miniracer.TotalWays		= 0;
	miniracer.FinishPlace	= 0;
	
	for (i=0; i<64; i++)
	{
		miniracer.waypoint[i].used		= false;
	}

	for (i=0; i<8; i++)
	{
		miniracer.startpoint[i].used	= false;
		miniracer.finishpoint[i].used	= false;
	}

	BotInit ();

	Precaches ();
}

void GO (void)
{
	float num;
	edict_t	*ent;

	ent = Nextent (miniracer.world);
	num = 0;

	while (num < miniracer.MaxClients)
	{
		if (ent->game.Active)
		{
			ent->game.laptime[0]	= sv.time;
			ent->game.ready			= true;
		}

		num += 1;
		ent = Nextent(ent);
	}
}

void ImpulseCommands (edict_t *ent)
{
	extern	cvar_t	langame;
	float			impulse = ent->game.impulse;

	if (impulse == 1)
	{
		float num;

		if (miniracer.started)
			return;

		MSG_WriteByte (&sv.reliable_datagram, svc_countdown);
		ent->game.nextthink = sv.time + 3;
		ent->game.think		= GO;

		ent = Nextent (miniracer.world);
		num = 0;

		if (!langame.ivalue)
		{
			while (num < miniracer.MaxClients)
			{
				if (!ent->game.Active) 
					BotConnect (ent);

				num += 1;
				ent = Nextent (ent);
			}
		}

		ent = Nextent (miniracer.world);
		num = 0;

		while (num < miniracer.MaxClients)
		{
			if (ent->game.Active && !ent->game.finished) 
			{
				ent->game.solid 	= SOLID_BBOX;
				ent->game.movetype 	= MOVETYPE_WALK;
			}

			num += 1;
			ent = Nextent (ent);
		}

		miniracer.started = true;
	}
	else if (impulse == 255)
	{
		Con_Printf ("%f\n", ent->game.laptime[1]);
		Con_Printf ("%f\n", ent->game.laptime[2]);
		Con_Printf ("%f\n", ent->game.laptime[3]);
	}
}

void RotateCar (void)
{
	edict_t	*ent;
	vec3_t	origin;
	float	distance;
	
	ent			= miniracer.self;

	origin[0] = ent->game.origin[0] - ent->game.goalorigin[0];
	origin[1] = ent->game.origin[1] - ent->game.goalorigin[1];
	origin[2] = ent->game.origin[2] - ent->game.goalorigin[2];

	distance = Length(origin);

	if(ent->v.angles[1] == ent->game.ideal_yaw && distance < 2)
	{
		ent->game.movetype	= MOVETYPE_NONE;
		return;
	}

	ent->game.velocity[0]	= 25 * host_frametime * (ent->game.goalorigin[0] - ent->game.origin[0]);
	ent->game.velocity[1]	= 25 * host_frametime * (ent->game.goalorigin[1] - ent->game.origin[1]);
	ent->game.velocity[2]	= 25 * host_frametime * (ent->game.goalorigin[2] - ent->game.origin[2]);

	ent->game.nextthink		= sv.time + 0.01;
	ent->game.think			= RotateCar;

	CalcYaw (ent);

	ChangeAngles (ent);
}

void GetCarToMove (edict_t *ent)
{
	float	max_forward_speed, max_back_speed;
	vec3_t	x, x1, x2, z, z1, z2, v;
	vec3_t	origin;
	vec3_t	origin2;

	if(ent->game.finished)
		return;

	if (ent->game.laptime[3])
	{
		ent->game.finished = true;
		RotateCar ();
		return;
	}

	if(ent->game.ready)
	{
		vec3_t	forward, right, up;
		trace_t	trace;

		max_forward_speed	= 600 + (ent->game.turbo * 400);
		max_back_speed		= 300;

		origin[0] = ent->game.origin[0];
		origin[1] = ent->game.origin[1];
		origin[2] = ent->game.origin[2] - 16;

		trace = Traceline (ent->game.origin, origin, ent);

		if (trace.fraction == 1)
			return;

		AngleVectors (ent->v.angles, forward, right, up);

		if (ent->game.brake)
		{
			ent->game.velocity[0] -= ent->game.velocity[0] * host_frametime * 3;
			ent->game.velocity[1] -= ent->game.velocity[1] * host_frametime * 3;
			ent->game.velocity[2] -= ent->game.velocity[2] * host_frametime * 3;
		}
		else if (ent->game.forward)
		{
			ent->game.velocity[0] += ((forward[0] * max_forward_speed) - ent->game.velocity[0]) * host_frametime;
			ent->game.velocity[1] += ((forward[1] * max_forward_speed) - ent->game.velocity[1]) * host_frametime;
			ent->game.velocity[2] += ((forward[2] * max_forward_speed) - ent->game.velocity[2]) * host_frametime;
		}
		else if (ent->game.back)
		{
			ent->game.velocity[0] -= ((forward[0] * max_back_speed) + ent->game.velocity[0]) * host_frametime * 0.4;
			ent->game.velocity[1] -= ((forward[1] * max_back_speed) + ent->game.velocity[1]) * host_frametime * 0.4;
			ent->game.velocity[2] -= ((forward[2] * max_back_speed) + ent->game.velocity[2]) * host_frametime * 0.4;
		}

		ent->game.velocity[0] -= ent->game.velocity[0] * host_frametime * 0.45;
		ent->game.velocity[1] -= ent->game.velocity[1] * host_frametime * 0.45;
		ent->game.velocity[2] -= ent->game.velocity[2] * host_frametime * 0.45;

		if (ent->game.left)
		{
			ent->game.yaw_speed += 800 * host_frametime;
		}
		else if (ent->game.right)
		{
			ent->game.yaw_speed -= 800 * host_frametime;
		}
		else
		{
			if (ent->game.yaw_speed > 0)
			{
				ent->game.yaw_speed -= 600 * host_frametime;
				if (ent->game.yaw_speed < 0) 
					ent->game.yaw_speed = 0;
			}
			if (ent->game.yaw_speed < 0)
			{
				ent->game.yaw_speed += 600 * host_frametime;
				if (ent->game.yaw_speed > 0) 
					ent->game.yaw_speed = 0;
			}
		}

		ChangeAngles (ent);

		forward[0]	*= 12;
		forward[1]	*= 12;
		forward[2]	*= 12;

		right[0]	*= 6;
		right[1]	*= 6;
		right[2]	*= 6;

		// first see which way the ground slopes in front of you
		origin[0] = ent->game.origin[0] + forward[0];
		origin[1] = ent->game.origin[1] + forward[1];
		origin[2] = ent->game.origin[2] + forward[2];

		origin2[0] = origin[0];
		origin2[1] = origin[1];
		origin2[2] = origin[2] - 16;

		trace = Traceline (origin, origin2, ent);
		if (trace.fraction == 1)
			return;
		VectorCopy (trace.endpos, x1);

		// second see which way the ground slopes behind you
		origin[0] = ent->game.origin[0] - forward[0];
		origin[1] = ent->game.origin[1] - forward[1];
		origin[2] = ent->game.origin[2] - forward[2];

		origin2[0] = origin[0];
		origin2[1] = origin[1];
		origin2[2] = origin[2] - 16;

		trace = Traceline (origin, origin2, ent);
		if (trace.fraction == 1)
			return;
		VectorCopy (trace.endpos, x2);

		// third see which way the ground slopes to the left
		origin[0] = ent->game.origin[0] - right[0];
		origin[1] = ent->game.origin[1] - right[1];
		origin[2] = ent->game.origin[2] - right[2];

		origin2[0] = origin[0];
		origin2[1] = origin[1];
		origin2[2] = origin[2] - 16;

		trace = Traceline (origin, origin2, ent);
		if (trace.fraction == 1)
			return;
		VectorCopy (trace.endpos, z1);

		// finally see which way the ground slopes to the right
		origin[0] = ent->game.origin[0] + right[0];
		origin[1] = ent->game.origin[1] + right[1];
		origin[2] = ent->game.origin[2] + right[2];

		origin2[0] = origin[0];
		origin2[1] = origin[1];
		origin2[2] = origin[2] - 16;

		trace = Traceline (origin, origin2, ent);	
		if (trace.fraction == 1)
			return;
		VectorCopy (trace.endpos, z2);

		VectorSubtract (x1, x2 ,x);
		VectorSubtract (z1, z2 ,z);

		Normalize (x, v);
		VectoAngles (v, v);
		ent->v.angles[0] = v[0];
	
		Normalize (z, v);
		VectoAngles (v, v);
		ent->v.angles[2] = v[0];
	}
}
