/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include	"../quakedef.h"
#include	"../sound.h"
#include	"../menu.h"
#include	"../vid.h"

typedef struct menu_items_s
{
	char		text[24];
	int			coords[4];
	int			*ivalue;
	float		*fvalue;
	char		**string;
	qboolean	type;
} menu_items_t;

typedef struct menu_s
{
	int				num_items;
	menu_items_t	items[20];
} menu_t;

enum 
{
	m_main,
	m_newgame,
	m_joingame,
	m_options,
	m_setupcar,
	m_setupmap,
	m_keys,
	m_sound,
	m_video,
	m_none
} m_state;

menu_t menus[10];

void M_Quit_f		(void);
void M_ColorSkin	(void);
void S_Play_2DSound	(byte wavnum, int volume);
void S_Play_Music	(int volume);
void S_Stop_Music	(void);

byte*		LoadMRTData (char* filename, qboolean complain);

cvar_t		car		= {"car",     "F40",true};
cvar_t		volume0	= {"volume0", "10", true};
cvar_t		volume1	= {"volume1", "10", true};
cvar_t		volume2	= {"volume2", "10", true};
cvar_t		volume3	= {"volume3", "10", true};
cvar_t		volume4	= {"volume4", "10", true};
cvar_t		volume5	= {"volume5", "10", true};
cvar_t		volume6	= {"volume6", "10", true};
list_t		themes[64];
list_t		cars[64];

int			cursor;

int			startmap = 1;
int			m_save_demonum;
int			car_texture;
int			bind_grab;
int			car_count=0;	//initialization for gcc under win32
int			theme_count=0;	//initialization for gcc under win32
int			current_car=0;	//initialization for gcc under win32
int			current_theme=0;//initialization for gcc under win32

char		join_ip[16];
char		clientname[16];

qboolean	m_entersound;
qboolean	scr_drawmenu;

extern	cvar_t	_timeofday;
extern	cvar_t	numbots;
extern	cvar_t	vid_brightness;
extern	cvar_t	vid_contrast;
extern	cvar_t	vid_gamma;
extern	cvar_t	draw_sky;
extern	cvar_t	draw_water;
extern	cvar_t	gl_skids;
extern	cvar_t	gl_smoke;
extern	cvar_t	gl_shadow;

extern	map_t	maps[6];

extern	int		image_width;
extern	int		image_height;

/*
=======
M_Print
=======
*/
void M_Print (int x, int y, char *str, int draw_cursor)
{
	if (draw_cursor == cursor)
	{
		char	*str2	= str;
		byte	i		= MR_sin (realtime * 6.28318530717958647692f) * 63 + 192;

		Draw_String (x, y, str2, 4, 0, 0, 255, i);
		Draw_String (x, y, str, 0, i, i, 255, 255);
	}
	else
		Draw_String (x, y, str, 0, 255, 255, 255, 255);
}

/*
======
M_Draw
======
*/
void M_Draw (void)
{
	int	i;

	for (i=0; i<menus[m_state].num_items; i++)
	{
		M_Print (menus[m_state].items[i].coords[0], menus[m_state].items[i].coords[2], menus[m_state].items[i].text, i);

		if (menus[m_state].items[i].type == 1)
			M_Print (320, menus[m_state].items[i].coords[2], *menus[m_state].items[i].string, i);
		else if (menus[m_state].items[i].type == 2)
			M_Print (320, menus[m_state].items[i].coords[2], va("%i", *menus[m_state].items[i].ivalue), i);
		else if (menus[m_state].items[i].type == 3)
			M_Print (320, menus[m_state].items[i].coords[2], va("%.1f", *menus[m_state].items[i].fvalue), i);
	}
}

/*
=====
M_Key
=====
*/
void M_Key (int key)
{
	switch (key)
	{
	case K_DOWN:
		S_Play_2DSound (0, volume0.ivalue);
		if (++cursor >= menus[m_state].num_items)
			cursor = 0;
		break;

	case K_UP:
		S_Play_2DSound (0, volume0.ivalue);
		if (--cursor < 0)
			cursor = menus[m_state].num_items - 1;
		break;
	}
}

/*
===
M_f
===
*/
void M_f (int state)
{
	if (key_dest != key_menu)
	{
		m_save_demonum	= cls.demonum;
		cls.demonum		= -1;
	}

	MR_strcpy (clientname, _cl_name.string);

	key_dest		= key_menu;
	m_state			= state;
	m_entersound	= true;
	cursor			= 0;
}

/*
==============
M_ToggleMenu_f
==============
*/
void M_ToggleMenu_f (void)
{
	M_f (m_main);
	startmap = 1;
}

//=============================================================================
/* MAIN MENU */

qboolean	mouse_visible = false;
qboolean	model_visible = false;

POINT		current_pos;
POINT		current_pos_old;
extern	int	CenterX, CenterY;

void M_Main_Draw (void)
{
}

void M_Main_Key (int key)
{
	switch (key)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		key_dest = key_game;
		m_state = m_none;
		cls.demonum = m_save_demonum;
		if (cls.demonum != -1 && !cls.demoplayback && cls.state != ca_connected)
			CL_NextDemo ();
		break;

	case K_MOUSE1:
	case K_ENTER:
		m_entersound = true;

		switch (cursor)
		{
		case 0:
			M_f (m_newgame);
			break;

		case 1:
			M_f (m_joingame);
			break;

		case 2:
			M_f (m_options);
			break;

		case 3:
			M_Quit_f ();
			break;
		}
	}
}

//=============================================================================
/* NEW GAME MENU */

void M_NewGame_Draw (void)
{
	int	x		= vid.width  * 0.5;
	int	y		= vid.height * 0.25;
	int	flash	= (int)(realtime*4)&1;


	if (cursor == 4)
	{
		M_Print (x - ((int)MR_strlen (clientname) * 6), y + 64,  clientname, -1);
		if (flash)
			Draw_Character (x + ((int)MR_strlen (clientname) * 6), y + 64, 10, 0, 255, 255, 255, 255);
		else
			Draw_Character (x + ((int)MR_strlen (clientname) * 6), y + 64, 12, 0, 255, 255, 255, 255);
	}
	else if (cursor == 5)
		//M_Print (x - 48, y + 64, "Disabled", -1);
		M_Print (x - 6, y + 64, va("%i", numbots.ivalue), -1);
}

void M_NewGame_Key (int key)
{
	int		l;

	switch (key)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_main);
		break;

	case K_LEFT:
		if ((cursor == 5) && (numbots.ivalue != 0))
		{
			S_Play_2DSound (0, (int)volume0.ivalue);
			numbots.ivalue--;

			if (numbots.ivalue < 0)
				numbots.ivalue = 0;

			Cvar_SetValue ("numbots", numbots.ivalue);
		}
		break;

	case K_RIGHT:
		if ((cursor == 5) && (numbots.ivalue != 7))
		{
			S_Play_2DSound (0, (int)volume0.ivalue);
			numbots.ivalue++;

			if (numbots.ivalue > 7)
				numbots.ivalue = 7;

			Cvar_SetValue ("numbots", numbots.ivalue);
		}
		break;

	case K_MOUSE1:
	case K_ENTER:
		if (cursor < 3)
			m_entersound = true;

		switch (cursor)
		{
		case 0:
			if (MR_strcmp (_cl_name.string, clientname) != 0)
				Cbuf_AddText (va("name %s\n", clientname));

			Cbuf_AddText ("disconnect\n");
			Cbuf_AddText ("listen 1\n");
			Cbuf_AddText ("langame 0\n");
			Cbuf_AddText (va("map %s\n", maps[startmap].name));
			break;

		case 1:
			if (MR_strcmp (_cl_name.string, clientname) != 0)
				Cbuf_AddText (va("name %s\n", clientname));

			Cbuf_AddText ("disconnect\n");
			Cbuf_AddText ("listen 1\n");
			Cbuf_AddText ("langame 1\n");
			Cbuf_AddText (va("map %s\n", maps[startmap].name));
			break;

		case 2:
			M_f (m_setupcar);
			M_ColorSkin ();
			break;

		case 3:
			M_f (m_setupmap);
			break;
		}
		break;

	case K_BACKSPACE:
		if (cursor == 4)
		{
			if (MR_strlen (clientname))
				clientname[MR_strlen (clientname)-1] = 0;
		}
		break;

	default:
		if (key < 32 || key > 127)
			break;

		if (cursor == 4)
		{
			l = MR_strlen (clientname);
			if (l < 15)
			{
				clientname[l+1] = 0;
				clientname[l] = key;
			}
		}
		break;
	}
}

//=============================================================================
/* JOIN GAME MENU */

void M_JoinGame_Draw (void)
{
	M_Print (320, 80, clientname, 1);

	M_Print (320, 144, join_ip, 4);
}

void M_JoinGame_Key (int key)
{
	int		l;
	
	switch (key)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_main);
		break;

	case K_MOUSE1:
	case K_ENTER:
		m_entersound = true;

		if (cursor == 0)
		{
			if (MR_strcmp (_cl_name.string, clientname) != 0)
				Cbuf_AddText (va("name %s\n", clientname) );

			key_dest	= key_game;
			m_state		= m_none;

			Cbuf_AddText (va("connect \"%s\"\n", join_ip));
		}
		break;

	case K_BACKSPACE:
		if (cursor == 1)
		{
			l = MR_strlen (clientname);
			if (l)
				clientname[l-1] = 0;
		}

		if (cursor == 4)
		{
			l = MR_strlen (join_ip);
			if (l)
				join_ip[l-1] = 0;
		}
		break;

	default:
		if (key < 32 || key > 127)
			break;

		if (cursor == 1)
		{
			l = MR_strlen (clientname);
			if (l < 15)
			{
				clientname[l+1] = 0;
				clientname[l]	= key;
			}
		}

		if (cursor == 4)
		{
			if ((key < 48 || key > 57) && (key != 46))
				break;

			l = MR_strlen (join_ip);
			if (l < 15)
			{
				join_ip[l+1]	= 0;
				join_ip[l]		= key;
			}
		}
	}
}

//=============================================================================
/* OPTIONS MENU */

void M_Options_Draw (void)
{
}

void M_Options_Key (int k)
{
	switch (k)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_main);
		break;

	case K_MOUSE1:
	case K_ENTER:
		m_entersound = true;
		switch (cursor)
		{
		case 0:
			M_f (m_keys);
			break;

		case 1:
			M_f (m_sound);
			break;

		case 2:
			M_f (m_video);
			break;

		case 3:
			m_state = m_none;
			Con_ToggleConsole_f ();
			break;

		case 4:
			Cbuf_AddText ("exec default.cfg\n");
			break;
		}
		break;
	}
}

//=============================================================================
/* CAR SETUP MENU */

void M_ColorSkin (void)
{
	int			i;
	byte		*temp, *data, *in, *out;

	data	= LoadMRTData (va("models/%s", car.string), true);

	temp	= malloc (image_width*image_height*4);
	in		= data;
	out		= temp;

	for (i = 0;i < image_width*image_height;i++)
	{
		if (in[0] == in[1] && in[0] == in[2]) // any shade of grey
		{
			out[0]	= in[0]	* _cl_color_red.ivalue		/ 256;
			out[1]	= in[1] * _cl_color_green.ivalue	/ 256;
			out[2]	= in[2]	* _cl_color_blue.ivalue		/ 256;
		}
		else
		{
			out[0]	= in[0];
			out[1]	= in[1];
			out[2]	= in[2];
		}

		out[3]		= in[3];

		in	+= 4;
		out += 4;
	}
	car_texture = GL_LoadTexture("car_texture", image_width, image_height, temp, true, false);

	free(data);
	free(temp);
}

void M_SetupCar_Draw (void)
{
	glDisable (GL_TEXTURE_2D);
	glColor4ub ((byte)_cl_color_red.ivalue, (byte)_cl_color_green.ivalue, (byte)_cl_color_blue.ivalue, 255);

	glBegin (GL_QUADS);
		glVertex2f (400, 80);
		glVertex2f (448, 80);
		glVertex2f (448, 128);
		glVertex2f (400, 128);
	glEnd ();

	glColor4ub (255,255,255,255);
	glEnable (GL_TEXTURE_2D);

	model_visible = true;
}

void M_SetupCar_Change (int dir)
{
	switch (cursor)
	{
	case 0:
		current_car += dir;

		if		(current_car < 0)				current_car = car_count - 1;
		else if	(current_car > car_count - 1)	current_car = 0;	

		Cvar_Set ("car", cars[current_car].name);

		M_ColorSkin ();
		break;

	case 1:
		_cl_color_red.ivalue = bound (0, dir * 5 + _cl_color_red.ivalue, 255);

		if (_cl_color_red.ivalue != (int)_cl_color_red.fvalue)
		{
			S_Play_2DSound (0, volume0.ivalue);
			Cvar_SetValue ("_cl_color_red", _cl_color_red.ivalue);
		}
		break;

	case 2:
		_cl_color_green.ivalue = bound (0, dir * 5 + _cl_color_green.ivalue, 255);

		if (_cl_color_green.ivalue != (int)_cl_color_green.fvalue)
		{
			S_Play_2DSound (0, volume0.ivalue);
			Cvar_SetValue ("_cl_color_green", _cl_color_green.ivalue);
		}
		break;

	case 3:
		_cl_color_blue.ivalue = bound (0, dir * 5 + _cl_color_blue.ivalue, 255);

		if (_cl_color_blue.ivalue != (int)_cl_color_blue.fvalue)
		{
			S_Play_2DSound (0, volume0.ivalue);
			Cvar_SetValue ("_cl_color_blue", _cl_color_blue.ivalue);
		}
		break;
	}
}

void M_SetupCar_Key (int key)
{
	switch (key)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_newgame);
		break;

	case K_LEFT:
		M_SetupCar_Change (-1);
		if (cursor == 1 ||
			cursor == 2 ||
			cursor == 3)
			Key_Event (K_LEFT, false);
		break;

	case K_RIGHT:
		M_SetupCar_Change (1);
		if (cursor == 1 ||
			cursor == 2 ||
			cursor == 3)
			Key_Event (K_RIGHT, false);
		break;

	case K_MOUSE1:
	case K_ENTER:
		if (cursor == 4)
		{
			_cl_color_red.ivalue	= (MR_rand () & 51) * 5;
			_cl_color_green.ivalue	= (MR_rand () & 51) * 5;
			_cl_color_blue.ivalue	= (MR_rand () & 51) * 5;
			
			Cvar_SetValue ("_cl_color_red",   _cl_color_red.ivalue);
			Cvar_SetValue ("_cl_color_green", _cl_color_green.ivalue);
			Cvar_SetValue ("_cl_color_blue",  _cl_color_blue.ivalue);

			M_ColorSkin ();
		}
		else if (cursor == 5)
			M_ColorSkin ();
		else
		{
			S_Play_2DSound (0, volume0.ivalue);
			M_SetupCar_Change (1);
		}
		break;
	}
}

//=============================================================================
/* MAP SETUP MENU */

void M_SetupMap_Draw (void)
{
	int		minutes;
	int		seconds;
	int		hundreds;

	if		(_timeofday.ivalue == 1)	M_Print (320, 48, "Evening", 0);
	else if (_timeofday.ivalue == 2)	M_Print (320, 48, "Night", 0);
	else if (_timeofday.ivalue == 3)	M_Print (320, 48, "Morning", 0);
	else								M_Print (320, 48, "Day", 0);

	M_Print (320, 64, themes[current_theme].name, 1);

	M_Print (320, 80, maps[startmap].description, 2);

	minutes		= maps[startmap].laprecord / 60;
	seconds		= maps[startmap].laprecord - (60 * minutes);
	hundreds	= ((maps[startmap].laprecord - (60 * minutes)) - seconds) * 100;

	M_Print (320, 128, va("%02i:%02i:%02i", minutes, seconds, hundreds), -1);

	M_Print (320, 144, maps[startmap].lapholder, -1);

	minutes		= maps[startmap].racerecord / 60;
	seconds		= maps[startmap].racerecord - (60 * minutes);
	hundreds	= ((maps[startmap].racerecord - (60 * minutes)) - seconds) * 100;

	M_Print (320, 176, va("%02i:%02i:%02i", minutes, seconds, hundreds), -1);

	M_Print (320, 192, maps[startmap].raceholder, -1);
}

void M_SetupMap_Change (int dir)
{
	switch (cursor)
	{
	case 0:
		_timeofday.ivalue += dir;

		if (_timeofday.ivalue < 0)	_timeofday.ivalue = 3;
		if (_timeofday.ivalue > 3)	_timeofday.ivalue = 0;

		Cvar_SetValue ("_timeofday", _timeofday.ivalue);
		break;

	case 1:
		current_theme += dir;

		if		(current_theme < 0)					current_theme = theme_count - 1;
		else if (current_theme > theme_count - 1)	current_theme = 0;
		break;

	case 2:
		startmap += dir;

		if (startmap < 1)	startmap = 5;
		if (startmap > 5)	startmap = 1;
		break;
	}
}

void M_SetupMap_Key (int key)
{
	switch (key)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_newgame);
		break;

	case K_LEFT:
		S_Play_2DSound (0, volume0.ivalue);
		M_SetupMap_Change (-1);
		break;

	case K_MOUSE1:
	case K_ENTER:
	case K_RIGHT:
		S_Play_2DSound (0, volume0.ivalue);
		M_SetupMap_Change (1);
		break;
	}
}

//=============================================================================
/* KEYS MENU */

char *bindnames[] =
{
	"+forward",
	"+back",
	"+left",
	"+right",
	"+brake",
	"+showtimes",
	"+cam1",
	"+cam2",
	"+cam3",
	"+camreset",
	"cameraup",
	"cameradown",
	"clientup",
	"clientdown",
	"impulse 1"
};

void M_FindKeysForCommand (char *command, int *twokeys)
{
	int		count;
	int		j;
	int		l;
	char	*b;

	twokeys[0] = twokeys[1] = -1;
	l = MR_strlen (command);
	count = 0;

	for (j=0 ; j<256 ; j++)
	{
		b = keybindings[j];
		if (!b)
			continue;
		if (!MR_strncmp (b, command, l) )
		{
			twokeys[count] = j;
			count++;
			if (count == 2)
				break;
		}
	}
}

void M_UnbindCommand (char *command)
{
	int		j;
	int		l;
	char	*b;

	l = MR_strlen (command);

	for (j=0 ; j<256 ; j++)
	{
		b = keybindings[j];
		if (!b)
			continue;
		if (!MR_strncmp (b, command, l) )
			Key_SetBinding (j, "");
	}
}

void M_Keys_Draw (void)
{
	int		i;
	int		keys[2];
	char	*name;
	int		x, y;

	if (bind_grab)
		M_Print (120, 16, "Press a key or button for this action", -2);
	else
		M_Print (120, 16, "Enter to change, backspace to clear", -2);

// search for known bindings
	for (i=0; i<15; i++)
	{
		y = 48 + 16*i;

		M_FindKeysForCommand (bindnames[i], keys);

		if (keys[0] == -1)
		{
			M_Print (336, y, "???", i);
		}
		else
		{
			name = Key_KeynumToString (keys[0]);
			M_Print (336, y, name, i);
			if (keys[1] != -1)
			{
				x = MR_strlen (name) * 12;
				M_Print (336 + x + 12, y, "or", i);
				M_Print (336 + x + 48, y, Key_KeynumToString (keys[1]), i);
			}
		}
	}

	if (bind_grab)	Draw_Character (320, 48 + cursor*16, '=', 0, 255, 255, 255, 255);
}

void M_Keys_Key (int k)
{
	char	cmd[80];
	int		keys[2];

	if (bind_grab)
	{	// defining a key
		S_Play_2DSound (0, volume0.ivalue);
		if (k == K_ESCAPE)
		{
			bind_grab = false;
		}
		else if (k != '`')
		{
			_snprintf (cmd, sizeof(cmd), "bind \"%s\" \"%s\"\n", Key_KeynumToString (k), bindnames[cursor]);
			Cbuf_InsertText (cmd);
		}

		bind_grab = false;
		return;
	}

	switch (k)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_options);
		break;

	case K_MOUSE1:
	case K_ENTER:		// go into bind mode
		M_FindKeysForCommand (bindnames[cursor], keys);
		S_Play_2DSound (1, volume1.ivalue);
		if (keys[1] != -1)
			M_UnbindCommand (bindnames[cursor]);
		bind_grab = true;
		break;

	case K_BACKSPACE:		// delete bindings
	case K_DEL:				// delete bindings
		S_Play_2DSound (1, volume1.ivalue);
		M_UnbindCommand (bindnames[cursor]);
		break;

	}
}

void M_Sound_Draw (void)
{
}

void M_Sound_Change (int dir)
{
	switch (cursor)
	{
	case 0:
		volume0.ivalue += dir;

		if (volume0.ivalue < 0)		volume0.ivalue = 0;
		if (volume0.ivalue > 10)	volume0.ivalue = 10;

		Cvar_SetValue ("volume0", volume0.ivalue);
		break;

	case 1:
		volume1.ivalue += dir;

		if (volume1.ivalue < 0)		volume1.ivalue = 0;
		if (volume1.ivalue > 10)	volume1.ivalue = 10;

		Cvar_SetValue ("volume1", volume1.ivalue);
		break;

	case 2:
		volume2.ivalue += dir;

		if (volume2.ivalue < 0)		volume2.ivalue = 0;
		if (volume2.ivalue > 10)	volume2.ivalue = 10;

		Cvar_SetValue ("volume2", volume2.ivalue);
		break;

	case 3:
		volume3.ivalue += dir;

		if (volume3.ivalue < 0)		volume3.ivalue = 0;
		if (volume3.ivalue > 10)	volume3.ivalue = 10;

		Cvar_SetValue ("volume3", volume3.ivalue);
		break;

	case 4:
		volume4.ivalue += dir;

		if (volume4.ivalue < 0)		volume4.ivalue = 0;
		if (volume4.ivalue > 10)	volume4.ivalue = 10;

		Cvar_SetValue ("volume4", volume4.ivalue);
		break;

	case 5:
		volume5.ivalue += dir;

		if (volume5.ivalue < 0)		volume5.ivalue = 0;
		if (volume5.ivalue > 10)	volume5.ivalue = 10;

		Cvar_SetValue ("volume5", volume5.ivalue);
		break;

	case 6:
		volume6.ivalue += dir;

		if (volume6.ivalue < 0)		volume6.ivalue = 0;
		if (volume6.ivalue > 10)	volume6.ivalue = 10;

		Cvar_SetValue ("volume6", volume6.ivalue);
		break;
	}
}

void M_Sound_Key (int k)
{
	switch (k)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_options);
		break;

	case K_MOUSE1:
	case K_ENTER:
		if (cursor == 7)
		{
			S_Stop_Music ();
			S_Play_Music (volume6.ivalue);
		}
		else
			M_Sound_Change (1);
		break;

		M_Sound_Change (1);
		break;

	case K_LEFT:
		M_Sound_Change (-1);
		break;

	case K_RIGHT:
		M_Sound_Change (1);
		break;
	}
}

void M_Video_Draw (void)
{
}

void M_Video_Change (int dir)
{
	switch (cursor)
	{
	case 0:
		Cvar_SetValue ("vid_brightness", vid_brightness.fvalue + dir*0.1);
		break;

	case 1:
		Cvar_SetValue ("vid_contrast", vid_contrast.fvalue + dir*0.1);
		break;

	case 2:
		Cvar_SetValue ("vid_gamma",	vid_gamma.fvalue + dir*0.1);
		break;

	case 3:
		Cvar_SetValue ("draw_sky", !draw_sky.ivalue);
		break;

	case 4:
		Cvar_SetValue ("draw_water", !draw_water.ivalue);
		break;

	case 5:
		Cvar_SetValue ("gl_skids", !gl_skids.ivalue);
		break;

	case 6:
		Cvar_SetValue ("gl_smoke", !gl_smoke.ivalue);
		break;

	case 7:
		Cvar_SetValue ("gl_shadow", gl_shadow.ivalue + dir);
		if (gl_shadow.ivalue < 0)
			Cvar_SetValue ("gl_shadow", 3);
		if (gl_shadow.ivalue > 3)
			Cvar_SetValue ("gl_shadow", 0);
		break;
	}
}

void M_Video_Key (int k)
{
	switch (k)
	{
	case K_MOUSE2:
	case K_ESCAPE:
		M_f (m_options);
		break;

	case K_MOUSE1:
	case K_ENTER:
		M_Video_Change (1);
		break;

	case K_LEFT:
		M_Video_Change (-1);
		break;

	case K_RIGHT:
		M_Video_Change (1);
		break;
	}
}

//=============================================================================
/* QUIT MENU */

void M_Quit_f (void)
{
	key_dest = key_console;
	Host_Quit_f ();
}

//=============================================================================
/* Menu Subsystem */

void Menu_Mouse (void)
{
	int	i;

	if (m_state == m_none)
		return;

	current_pos_old.x = current_pos.x;
	current_pos_old.y = current_pos.y;

	GetCursorPos (&current_pos);

	current_pos.x -= CenterX;
	current_pos.y -= CenterY;

	if (current_pos.x < 0)
	{
		if (current_pos.y < 0)
			SetCursorPos (CenterX, CenterY);
		else
			SetCursorPos (CenterX, CenterY + current_pos.y);
	}
	else if (current_pos.y < 0)
		SetCursorPos (CenterX + current_pos.x, CenterY);

	if (current_pos.x > (signed)vid.width)
	{
		if (current_pos.y > (signed)vid.height)
			SetCursorPos (CenterX + vid.width, CenterY + vid.height);
		else
			SetCursorPos (CenterX + vid.width, CenterY + current_pos.y);
	}
	else if (current_pos.y > (signed)vid.height)
		SetCursorPos (CenterX + current_pos.x, CenterY + vid.height);

	if (!mouse_visible)
	{
		if (current_pos_old.x != current_pos.x || current_pos_old.y != current_pos.y)
			mouse_visible = true;
		else
			return;
	}

	cursor = -1;

	for (i=0; i<menus[m_state].num_items; i++)
		if (current_pos.x > menus[m_state].items[i].coords[0] &&
			current_pos.x < menus[m_state].items[i].coords[1] &&
			current_pos.y > menus[m_state].items[i].coords[2] &&
			current_pos.y < menus[m_state].items[i].coords[3])
			cursor = i;
}

void SCR_DrawMenu (void)
{
	if (m_state == m_none || key_dest != key_menu)
	{
		scr_drawmenu = false;
		return;
	}

	model_visible = false;

	scr_dest = scr_menu;
	scr_drawmenu = true;

	Draw_ConsoleBackground ();

	if (m_state != m_none)
		M_Draw ();

	switch (m_state)
	{
	case m_none:
		break;

	case m_main:
		M_Main_Draw ();
		break;

	case m_newgame:
		M_NewGame_Draw ();
		break;

	case m_setupcar:
		M_SetupCar_Draw ();
		break;

	case m_setupmap:
		M_SetupMap_Draw ();
		break;

	case m_joingame:
		M_JoinGame_Draw ();
		break;

	case m_options:
		M_Options_Draw ();
		break;

	case m_keys:
		M_Keys_Draw ();
		break;

	case m_sound:
		M_Sound_Draw ();
		break;

	case m_video:
		M_Video_Draw ();
		break;
	}

	if (mouse_visible)
		Draw_Character (current_pos.x - 6, current_pos.y - 8, 'X', 0, 0, 0, 255, 255);

	if (m_entersound)
	{
		S_Play_2DSound (1, volume1.ivalue);
		m_entersound = false;
	}
}

void M_Keydown (int key)
{
	if (key != K_MOUSE1 && key != K_MOUSE2)
		mouse_visible = false;

	switch (m_state)
	{
	case m_none:
		break;

	case m_main:
		M_Main_Key (key);
		break;

	case m_newgame:
		M_NewGame_Key (key);
		break;

	case m_setupcar:
		M_SetupCar_Key (key);
		break;

	case m_setupmap:
		M_SetupMap_Key (key);
		break;

	case m_joingame:
		M_JoinGame_Key (key);
		break;

	case m_options:
		M_Options_Key (key);
		break;

	case m_keys:
		M_Keys_Key (key);
		break;

	case m_sound:
		M_Sound_Key (key);
		break;

	case m_video:
		M_Video_Key (key);
		break;
	}

	if (m_state != m_none)
		M_Key (key);
}

void SetupMenuItem (int menu, int item, char *text, int x, int y, int type, char **string, int *ivalue, float *fvalue)
{
	_snprintf (menus[menu].items[item].text, sizeof(menus[menu].items[item].text), text);
	menus[menu].items[item].coords[0]	= x;
	menus[menu].items[item].coords[1]	= x + (MR_strlen (text) * 12);
	menus[menu].items[item].coords[2]	= y;
	menus[menu].items[item].coords[3]	= y + 16;
	menus[menu].items[item].type		= type;
	menus[menu].items[item].string		= string;
	menus[menu].items[item].ivalue		= ivalue;
	menus[menu].items[item].fvalue		= fvalue;

	menus[menu].num_items++;
}
void SetupMenus (void)
{
	// Main Menu
	SetupMenuItem (0, 0,  "New Game",			120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (0, 1,  "Join Game",			120, 64,  0, NULL, NULL, NULL);
	SetupMenuItem (0, 2,  "Options",			120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (0, 3,  "Quit",				120, 96,  0, NULL, NULL, NULL);

	// New Game Menu
	SetupMenuItem (1, 0,  "Start Single Race",	120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (1, 1,  "Start Multi Race",	120, 64,  0, NULL, NULL, NULL);
	SetupMenuItem (1, 2,  "Car Setup",			120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (1, 3,  "Map Setup",			120, 96,  0, NULL, NULL, NULL);
	SetupMenuItem (1, 4,  "Name",				120, 112, 0, NULL, NULL, NULL);
	SetupMenuItem (1, 5,  "Bots",				120, 128, 0, NULL, NULL, NULL);

	// Join Game Menu
	SetupMenuItem (2, 0,  "Join Race",			120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (2, 1,  "Name",				120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (2, 2,  "Car Color",			120, 96,  0, NULL, NULL, NULL);
	SetupMenuItem (2, 3,  "Car Model",			120, 112, 0, NULL, NULL, NULL);
	SetupMenuItem (2, 4,  "IP To Join",			120, 144, 0, NULL, NULL, NULL);

	// Options Menu
	SetupMenuItem (3, 0,  "Controls",			120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (3, 1,  "Sound",				120, 64,  0, NULL, NULL, NULL);
	SetupMenuItem (3, 2,  "Video",				120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (3, 3,  "Go To Console",		120, 112, 0, NULL, NULL, NULL);
	SetupMenuItem (3, 4,  "Reset To Defaults",	120, 128, 0, NULL, NULL, NULL);

	// Car Setup Menu
	SetupMenuItem (4, 0,  "Model",				120, 48,  1, &car.string, NULL, NULL);
	SetupMenuItem (4, 1,  "Red",				120, 80,  2, NULL, &_cl_color_red.ivalue,	NULL);
	SetupMenuItem (4, 2,  "Green",				120, 96,  2, NULL, &_cl_color_green.ivalue,	NULL);
	SetupMenuItem (4, 3,  "Blue",				120, 112, 2, NULL, &_cl_color_blue.ivalue,	NULL);
	SetupMenuItem (4, 4,  "Random Color",		120, 128, 0, NULL, NULL, NULL);
	SetupMenuItem (4, 5,  "Apply Color",		120, 144, 0, NULL, NULL, NULL);

	// Map Setup Menu
	SetupMenuItem (5, 0,  "Time",				120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (5, 1,  "Scene",				120, 64,  0, NULL, NULL, NULL);
	SetupMenuItem (5, 2,  "Map",				120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (5, 3,  "Lap Record",			120, 128, 0, NULL, NULL, NULL);
	SetupMenuItem (5, 4,  "Record Holder",		120, 144, 0, NULL, NULL, NULL);
	SetupMenuItem (5, 5,  "Race Record",		120, 176, 0, NULL, NULL, NULL);
	SetupMenuItem (5, 6,  "Record Holder",		120, 192, 0, NULL, NULL, NULL);

	// Key Setup Menu
	SetupMenuItem (6, 0,  "Accelerate",			120, 48,  0, NULL, NULL, NULL);
	SetupMenuItem (6, 1,  "Reverse",			120, 64,  0, NULL, NULL, NULL);
	SetupMenuItem (6, 2,  "Turn Left",			120, 80,  0, NULL, NULL, NULL);
	SetupMenuItem (6, 3,  "Turn Right",			120, 96,  0, NULL, NULL, NULL);
	SetupMenuItem (6, 4,  "Brake",				120, 112, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 5,  "Show Times",			120, 128, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 6,  "Camera Control 1",	120, 144, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 7,  "Camera Control 2",	120, 160, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 8,  "Camera Control 3",	120, 176, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 9,  "Camera Reseter",		120, 192, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 10, "Cameramode Up",		120, 208, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 11, "Cameramode Down",	120, 224, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 12, "Chaseclient Up",		120, 240, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 13, "Chaseclient Down",	120, 256, 0, NULL, NULL, NULL);
	SetupMenuItem (6, 14, "Start Race",			120, 272, 0, NULL, NULL, NULL);

	// Sound Setup Menu
	SetupMenuItem (7, 0,  "Menu1  Volume",		120, 48,  2, NULL, &volume0.ivalue, NULL);
	SetupMenuItem (7, 1,  "Menu2  Volume",		120, 64,  2, NULL, &volume1.ivalue, NULL);
	SetupMenuItem (7, 2,  "Beep1  Volume",		120, 80,  2, NULL, &volume2.ivalue, NULL);
	SetupMenuItem (7, 3,  "Beep2  Volume",		120, 96,  2, NULL, &volume3.ivalue, NULL);
	SetupMenuItem (7, 4,  "Skid   Volume",		120, 112, 2, NULL, &volume4.ivalue, NULL);
	SetupMenuItem (7, 5,  "Engine Volume",		120, 128, 2, NULL, &volume5.ivalue, NULL);
	SetupMenuItem (7, 6,  "Music  Volume",		120, 144, 2, NULL, &volume6.ivalue, NULL);
	SetupMenuItem (7, 7,  "Change Music",		120, 160, 0, NULL, NULL, NULL);

	// Video Setup Menu
	SetupMenuItem (8, 0,  "Brightness",			120, 48,  3, NULL, NULL, &vid_brightness.fvalue);
	SetupMenuItem (8, 1,  "Contrast",			120, 64,  3, NULL, NULL, &vid_contrast.fvalue);
	SetupMenuItem (8, 2,  "Gamma",				120, 80,  3, NULL, NULL, &vid_gamma.fvalue);
	SetupMenuItem (8, 3,  "Draw Sky",			120, 96,  2, NULL, &draw_sky.ivalue, NULL);
	SetupMenuItem (8, 4,  "Draw Water",			120, 112, 2, NULL, &draw_water.ivalue, NULL);
	SetupMenuItem (8, 5,  "Skidmarks",			120, 128, 2, NULL, &gl_skids.ivalue, NULL);
	SetupMenuItem (8, 6,  "Smoke",				120, 144, 2, NULL, &gl_smoke.ivalue, NULL);
	SetupMenuItem (8, 7,  "Shadows",			120, 160, 2, NULL, &gl_shadow.ivalue, NULL);
}

void Menu_Init (void)
{
	Cvar_RegisterVariable (&car);
	Cvar_RegisterVariable (&volume0);
	Cvar_RegisterVariable (&volume1);
	Cvar_RegisterVariable (&volume2);
	Cvar_RegisterVariable (&volume3);
	Cvar_RegisterVariable (&volume4);
	Cvar_RegisterVariable (&volume5);
	Cvar_RegisterVariable (&volume6);

	SetupMenus ();

	m_state		= m_main;
	key_dest	= key_menu;

	theme_count	= LoadList ("textures/themes.lst", themes);

	car_count	= LoadList ("models/cars.lst", cars);
}
