/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"

#define MAX_SKIDS 2048

extern	cvar_t	gl_skids;

float	skid[MAX_SKIDS][4][3];

int		currentskid;

void GL_ClearSkids (void)
{
	currentskid = 0;

	MR_memset (&skid[0][0][0], 0, sizeof(skid));
}

void GL_SetupSkids(vec3_t neworg1, vec3_t oldorg1, vec3_t neworg2, vec3_t oldorg2)
{
	vec3_t	skidangle, forward, right, up, dist;

	if (!gl_skids.ivalue)
		return;

	VectorSubtract (oldorg1, neworg1, dist);
	VectorNormalize (dist);

	skidangle[0] = -(RAD2DEG(MR_asin (dist[2])));
	skidangle[1] = RAD2DEG(MR_atan2 (dist[1], dist[0]));
	skidangle[2] = 0;

	AngleVectors (skidangle, forward, right, up);	

	currentskid++;

	if (currentskid >= MAX_SKIDS)
		currentskid = 0;

	VectorAdd		(neworg1, right, skid[currentskid][0]);	// texcoords 0 1
	VectorSubtract	(neworg1, right, skid[currentskid][1]);	// texcoords 0 0
	VectorSubtract	(oldorg1, right, skid[currentskid][2]);	// texcoords 1 0
	VectorAdd		(oldorg1, right, skid[currentskid][3]);	// texcoords 1 1

	currentskid++;

	if (currentskid >= MAX_SKIDS)
		currentskid = 0;

	VectorAdd		(neworg2, right, skid[currentskid][0]);	// texcoords 0 1
	VectorSubtract	(neworg2, right, skid[currentskid][1]);	// texcoords 0 0
	VectorSubtract	(oldorg2, right, skid[currentskid][2]);	// texcoords 1 0
	VectorAdd		(oldorg2, right, skid[currentskid][3]);	// texcoords 1 1
}

#ifdef _WIN32
extern void	(APIENTRY *MRglLockArraysEXT)		(GLint first, GLint count);
extern void	(APIENTRY *MRglUnlockArraysEXT)	(void);
#else
extern void	*MRglLockArraysEXT		(GLint first, GLint count);
extern void	*MRglUnlockArraysEXT	(void);
#endif


void GL_DrawSkids (void)
{
	if (!gl_skids.ivalue)
		return;

	glDisable	(GL_TEXTURE_2D);
	glColor4ub	(0,0,0,64);
	glEnable	(GL_BLEND);
	glDepthMask	(false);

	glVertexPointer (3, GL_FLOAT, 12, &skid[0][0][0]);

	glEnableClientState (GL_VERTEX_ARRAY);

	if(MRglLockArraysEXT)
		MRglLockArraysEXT (0, MAX_SKIDS * 4);

	glDrawArrays (GL_QUADS, 0, MAX_SKIDS * 4);

	if (MRglUnlockArraysEXT)
		MRglUnlockArraysEXT ();

	glDisableClientState (GL_VERTEX_ARRAY);

	glDepthMask	(true);
	glDisable	(GL_BLEND);
	glColor4ub	(255,255,255,255);
	glEnable	(GL_TEXTURE_2D);
}

