/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"

#define MAX_SMOKE 1024

extern	cvar_t	gl_smoke;

typedef struct smoke_s
{
	float		scale;
	float		alpha;
	vec3_t		origin;
	vec3_t		velocity;
	byte		color[3];
	qboolean	used;
} smoke_t;

smoke_t smoke[MAX_SMOKE];

int		currentsmoke;
int		usedsmokes;
int		randomized;

float	smoke_texco[MAX_SMOKE][4][2];
float	smoke_coord[MAX_SMOKE][4][3];

byte	smoke_color[MAX_SMOKE][4][4];

extern	byte	lightcolor[3];

#ifdef _WIN32
extern	void	(APIENTRY *MRglLockArraysEXT)		(GLint first, GLint count);
extern	void	(APIENTRY *MRglUnlockArraysEXT)	(void);
#else
extern	void	*MRglLockArraysEXT		(GLint first, GLint count);
extern	void	*MRglUnlockArraysEXT	(void);
#endif

void GL_ClearSmoke (void)
{
	int	i;

	for (i=0; i<MAX_SMOKE; i++)
	{
		smoke_texco[i][0][0] = smoke_texco[i][0][1] = smoke_texco[i][1][1] = smoke_texco[i][3][0] = 0;
		smoke_texco[i][1][0] = smoke_texco[i][2][0] = smoke_texco[i][2][1] = smoke_texco[i][3][1] = 1;
	}

	MR_memset (&smoke[0],			  0, sizeof(smoke));
	MR_memset (&smoke_coord[0][0][0], 0, sizeof(smoke_coord));
	MR_memset (&smoke_color[0][0][0], 0, sizeof(smoke_color));
}

void GL_SetupSmoke (vec3_t origin1, vec3_t origin2, vec3_t forward, entity_t *ent)
{
	if (!gl_smoke.ivalue)
		return;

	R_LightPoint (ent->origin);

	currentsmoke++;

	if (currentsmoke == MAX_SMOKE)
		currentsmoke = 0;

	VectorCopy	(lightcolor, smoke[currentsmoke].color);
	VectorMA	(origin1, randomized, forward, smoke[currentsmoke].origin);

	smoke[currentsmoke].used		= true;
	smoke[currentsmoke].scale		= 1;
	smoke[currentsmoke].alpha		= 255;
	smoke[currentsmoke].velocity[0]	= -forward[0] * 16;
	smoke[currentsmoke].velocity[1]	= -forward[1] * 16;
	smoke[currentsmoke].velocity[2]	= 8;

	currentsmoke++;

	randomized = (MR_rand () & 7) - 4;

	if (currentsmoke == MAX_SMOKE)
		currentsmoke = 0;

	VectorCopy  (lightcolor, smoke[currentsmoke].color);
	VectorMA	(origin2, randomized, forward, smoke[currentsmoke].origin);

	smoke[currentsmoke].used		= true;
	smoke[currentsmoke].scale		= 1;
	smoke[currentsmoke].alpha		= 255;
	smoke[currentsmoke].velocity[0]	= -forward[0] * 16;
	smoke[currentsmoke].velocity[1]	= -forward[1] * 16;
	smoke[currentsmoke].velocity[2]	= 8;
}

void GL_DrawSmoke (void)
{
	extern	int		smoke_texture;

	int			i;

	float		frametime	= cl.time - cl.oldtime;
	float		ft24		= frametime * 24;
	float		ft256		= frametime * 256;

	vec3_t		up, right;

	if (!gl_smoke.ivalue)
		return;

	usedsmokes = 0;

	for (i=0; i<MAX_SMOKE; i++)
	{
		if (smoke[i].used)
		{
			smoke[i].scale	+= ft24;
			smoke[i].alpha	-= ft256;

			if(smoke[i].alpha < 16)
			{
				smoke[i].used = false;
				continue;
			}

			VectorMA	(smoke[i].origin,	frametime,		smoke[i].velocity,	smoke[i].origin);
			VectorScale	(vup,				smoke[i].scale,	up);
			VectorScale	(vright,			smoke[i].scale,	right);
			VectorCopy	(smoke[i].color,	smoke_color[usedsmokes][0]);
			VectorCopy	(smoke[i].color,	smoke_color[usedsmokes][1]);
			VectorCopy	(smoke[i].color,	smoke_color[usedsmokes][2]);
			VectorCopy	(smoke[i].color,	smoke_color[usedsmokes][3]);

			smoke_color[usedsmokes][0][3]	=
			smoke_color[usedsmokes][1][3]	=
			smoke_color[usedsmokes][2][3]	=
			smoke_color[usedsmokes][3][3]	= (byte)smoke[i].alpha;

			smoke_coord[usedsmokes][0][0]	= (smoke[i].origin[0] - right[0] + up[0]);
			smoke_coord[usedsmokes][0][1]	= (smoke[i].origin[1] - right[1] + up[1]);
			smoke_coord[usedsmokes][0][2]	= (smoke[i].origin[2] - right[2] + up[2]);

			smoke_coord[usedsmokes][1][0]	= (smoke[i].origin[0] + right[0] + up[0]);
			smoke_coord[usedsmokes][1][1]	= (smoke[i].origin[1] + right[1] + up[1]);
			smoke_coord[usedsmokes][1][2]	= (smoke[i].origin[2] + right[2] + up[2]);

			smoke_coord[usedsmokes][2][0]	= (smoke[i].origin[0] + right[0] - up[0]);
			smoke_coord[usedsmokes][2][1]	= (smoke[i].origin[1] + right[1] - up[1]);
			smoke_coord[usedsmokes][2][2]	= (smoke[i].origin[2] + right[2] - up[2]);

			smoke_coord[usedsmokes][3][0]	= (smoke[i].origin[0] - right[0] - up[0]);
			smoke_coord[usedsmokes][3][1]	= (smoke[i].origin[1] - right[1] - up[1]);
			smoke_coord[usedsmokes][3][2]	= (smoke[i].origin[2] - right[2] - up[2]);

			usedsmokes++;
		}
	}

	glBindTexture (GL_TEXTURE_2D, smoke_texture);
	glEnable	  (GL_BLEND);
	glTexEnvi	  (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glDepthMask	  (false);

	glTexCoordPointer (2, GL_FLOAT,			8,	&smoke_texco[0][0][0]);
	glVertexPointer	  (3, GL_FLOAT,			12,	&smoke_coord[0][0][0]);
	glColorPointer	  (4, GL_UNSIGNED_BYTE,	4,	&smoke_color[0][0][0]);

	glEnableClientState (GL_TEXTURE_COORD_ARRAY);
	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_COLOR_ARRAY);

	if (MRglLockArraysEXT)
		MRglLockArraysEXT (0, usedsmokes * 4);

	glDrawArrays (GL_QUADS, 0, usedsmokes * 4);

	if (MRglUnlockArraysEXT)
		MRglUnlockArraysEXT ();

	glDisableClientState (GL_TEXTURE_COORD_ARRAY);
	glDisableClientState (GL_VERTEX_ARRAY);
	glDisableClientState (GL_COLOR_ARRAY);

	glColor4ub	(255,255,255,255);
	glDepthMask	(true);
	glTexEnvi	(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glDisable	(GL_BLEND);
}

