/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/vt.h>
#include <stdarg.h>
#include <stdio.h>
#include <signal.h>

#include <dlfcn.h>

#include <GL/glx.h>

#include <X11/keysym.h>
#include <X11/cursorfont.h>

#include <X11/extensions/xf86dga.h>
#include <X11/extensions/xf86vmode.h>

// includes from gl_vidnt.c
#include "../quakedef.h"
#include "../vid.h"

// POINT structure for mouse handler
#include "../menu.h"

// declarations from gl_vidnt.c

#define         TEXTURE0_SGIS   0x835E
#define         TEXTURE1_SGIS   0x835F

#define         TEXTURE0_ARB    0x84C0
#define         TEXTURE1_ARB    0x84C1

GLenum          TEXTURE0 = 0;
GLenum          TEXTURE1 = 0;

MR_mode_t	mode;

viddef_t	vid;

short		sysgamma[3][256];
short		curgamma[3][256];

int		CenterX;
int		CenterY;

void	BuildGammaTable			(float gamma, float scale, float base, unsigned short *out);
void	VID_UpdateGamma			(qboolean force);
void	VID_RestoreSystemGamma		(void); 
// end declarations from gl_vidnt.c

// from WinQuake and gl_vidnt.c
typedef void (*lpMTexFUNC) (GLenum, GLfloat, GLfloat);
typedef void (*lpSelTexFUNC) (GLenum);
typedef void (*lpArraysLockFUNC) (GLint, GLint);
typedef void (*lpArraysUnlockFUNC) (void);
lpMTexFUNC MR1glMTexCoord2f;
lpSelTexFUNC MR1glSelectTexture;
lpArraysLockFUNC MR1glLockArraysEXT;
lpArraysUnlockFUNC MR1glUnlockArraysEXT;

static Display *dpy = NULL;
static int scrnum;
static Window win;
static XVisualInfo *visinfo;
static GLXContext ctx = NULL;

#define KEY_MASK (KeyPressMask | KeyReleaseMask)
#define MOUSE_MASK (ButtonPressMask | ButtonReleaseMask | \
		    PointerMotionMask | ButtonMotionMask )
#define X_MASK (KEY_MASK | MOUSE_MASK | VisibilityChangeMask | StructureNotifyMask )

cvar_t	vid_mode = {"vid_mode","0",false};
 
static qboolean        mouse_active;
 
qboolean vidmode_ext = false;

static int win_x, win_y;

static int scr_width, scr_height;

static XF86VidModeModeInfo **vidmodes;
static int num_vidmodes;
static qboolean vidmode_active = false;

/*-----------------------------------------------------------------------*/

//int		texture_mode = GL_NEAREST;
//int		texture_mode = GL_NEAREST_MIPMAP_NEAREST;
//int		texture_mode = GL_NEAREST_MIPMAP_LINEAR;
int		texture_mode = GL_LINEAR;
//int		texture_mode = GL_LINEAR_MIPMAP_NEAREST;
//int		texture_mode = GL_LINEAR_MIPMAP_LINEAR;

int		texture_extension_number = 1;

float		gldepthmin, gldepthmax;

const char *gl_vendor;
const char *gl_renderer;
const char *gl_version;
const char *gl_extensions;

qboolean gl_mtexable = false;
qboolean gl_mvertex = false;

/*-----------------------------------------------------------------------*/
static int XLateKey(XKeyEvent *ev)
{

	int key;
	char buf[64];
	KeySym keysym;

	key = 0;

	XLookupString(ev, buf, sizeof buf, &keysym, 0);

	switch(keysym)
	{
		case XK_KP_Page_Up:	 
		case XK_Page_Up:	 key = K_PGUP; break;

		case XK_KP_Page_Down: 
		case XK_Page_Down:	 key = K_PGDN; break;

		case XK_KP_Home: 
		case XK_Home:	 key = K_HOME; break;

		case XK_KP_End:  
		case XK_End:	 key = K_END; break;

		case XK_KP_Left: 
		case XK_Left:	 key = K_LEFT; break;

		case XK_KP_Right: 
		case XK_Right:	key = K_RIGHT;		break;

		case XK_KP_Down: 
		case XK_Down:	 key = K_DOWN; break;

		case XK_KP_Up:   
		case XK_Up:		 key = K_UP;	 break;

		case XK_Escape: key = K_ESCAPE;		break;

		case XK_KP_Enter: 
		case XK_Return: key = K_ENTER;		 break;

		case XK_Tab:		key = K_TAB;			 break;

		case XK_F1:		 key = K_F1;				break;

		case XK_F2:		 key = K_F2;				break;

		case XK_F3:		 key = K_F3;				break;

		case XK_F4:		 key = K_F4;				break;

		case XK_F5:		 key = K_F5;				break;

		case XK_F6:		 key = K_F6;				break;

		case XK_F7:		 key = K_F7;				break;

		case XK_F8:		 key = K_F8;				break;

		case XK_F9:		 key = K_F9;				break;

		case XK_F10:		key = K_F10;			 break;

		case XK_F11:		key = K_F11;			 break;

		case XK_F12:		key = K_F12;			 break;

		case XK_BackSpace: key = K_BACKSPACE; break;

		case XK_KP_Delete: 
		case XK_Delete: key = K_DEL; break;

		case XK_Pause:	key = K_PAUSE;		 break;

		case XK_Shift_L:
		case XK_Shift_R:	key = K_SHIFT;		break;

		case XK_Execute: 
		case XK_Control_L: 
		case XK_Control_R:	key = K_CTRL;		 break;

		case XK_Alt_L:	
		case XK_Meta_L: 
		case XK_Alt_R:	
		case XK_Meta_R: key = K_ALT;			break;

		case XK_KP_Begin: key = '5';	break;

		case XK_KP_Insert: 
		case XK_Insert:key = K_INS; break;

		case XK_KP_Multiply: key = '*'; break;
		case XK_KP_Add:  key = '+'; break;
		case XK_KP_Subtract: key = '-'; break;
		case XK_KP_Divide: key = '/'; break;

#if 0
		case 0x021: key = '1';break;/* [!] */
		case 0x040: key = '2';break;/* [@] */
		case 0x023: key = '3';break;/* [#] */
		case 0x024: key = '4';break;/* [$] */
		case 0x025: key = '5';break;/* [%] */
		case 0x05e: key = '6';break;/* [^] */
		case 0x026: key = '7';break;/* [&] */
		case 0x02a: key = '8';break;/* [*] */
		case 0x028: key = '9';;break;/* [(] */
		case 0x029: key = '0';break;/* [)] */
		case 0x05f: key = '-';break;/* [_] */
		case 0x02b: key = '=';break;/* [+] */
		case 0x07c: key = '\'';break;/* [|] */
		case 0x07d: key = '[';break;/* [}] */
		case 0x07b: key = ']';break;/* [{] */
		case 0x022: key = '\'';break;/* ["] */
		case 0x03a: key = ';';break;/* [:] */
		case 0x03f: key = '/';break;/* [?] */
		case 0x03e: key = '.';break;/* [>] */
		case 0x03c: key = ',';break;/* [<] */
#endif

		default:
			key = *(unsigned char*)buf;
			if (key >= 'A' && key <= 'Z')
				key = key - 'A' + 'a';
			break;
	} 

	return key;
}

static Cursor CreateNullCursor(Display *display, Window root)
{
    Pixmap cursormask; 
    XGCValues xgc;
    GC gc;
    XColor dummycolour;
    Cursor cursor;

    cursormask = XCreatePixmap(display, root, 1, 1, 1/*depth*/);
    xgc.function = GXclear;
    gc =  XCreateGC(display, cursormask, GCFunction, &xgc);
    XFillRectangle(display, cursormask, gc, 0, 0, 1, 1);
    dummycolour.pixel = 0;
    dummycolour.red = 0;
    dummycolour.flags = 04;
    cursor = XCreatePixmapCursor(display, cursormask, cursormask,
          &dummycolour,&dummycolour, 0,0);
    XFreePixmap(display,cursormask);
    XFreeGC(display,gc);
    return cursor;
}

static void install_grabs(void)
{
	/* vars to make blank cursor */
	Pixmap blank;
	XColor dummy;
	char data[1] = {0};
	Cursor cursor;
	/* make a blank cursor */
	blank = XCreateBitmapFromData (dpy, win, data, 1, 1);
	if(blank == None) fprintf(stderr, "error: out of memory.\n");
	cursor = XCreatePixmapCursor(dpy, blank, blank, &dummy, &dummy, 0, 0);
	XFreePixmap (dpy, blank);
	//Con_Printf("Cursor blanked.\n");

// inviso cursor
	XDefineCursor(dpy, win, CreateNullCursor(dpy, win));

	XGrabPointer(dpy, win,
				 True,
				 0,
				 GrabModeAsync, GrabModeAsync,
				 win,
				 cursor,
				 CurrentTime);

	XWarpPointer(dpy, None, win,
				 0, 0, 0, 0,
				 vid.width / 2, vid.height / 2);

	// grab keyboard only if we are in fullscreen mode
	if(mode.fullscreen)
	    XGrabKeyboard(dpy, win,
				  False,
				  GrabModeAsync, GrabModeAsync,
				  CurrentTime);

	mouse_active = false;

//	XSync(dpy, True);
}

static void uninstall_grabs(void)
{
	if (!dpy || !win)
		return;

	XUngrabPointer(dpy, CurrentTime);
	XUngrabKeyboard(dpy, CurrentTime);

// inviso cursor
	XUndefineCursor(dpy, win);

	mouse_active = false;
}

static void HandleEvents(void)
{
	XEvent event;
	int b;
	
	if (!dpy)
		return;

	while (XPending(dpy)) {
		XNextEvent(dpy, &event);

		switch (event.type) {
		case KeyPress:
		case KeyRelease:
			Key_Event(XLateKey(&event.xkey), event.type == KeyPress);
			break;

			break;

		case ButtonPress:
			b=-1;
			if (event.xbutton.button == 1)
				b = 0;
			else if (event.xbutton.button == 2)
				b = 2;
			else if (event.xbutton.button == 3)
				b = 1;
			if (b>=0)
				Key_Event(K_MOUSE1 + b, true);
			break;

		case ButtonRelease:
			b=-1;
			if (event.xbutton.button == 1)
				b = 0;
			else if (event.xbutton.button == 2)
				b = 2;
			else if (event.xbutton.button == 3)
				b = 1;
			if (b>=0)
				Key_Event(K_MOUSE1 + b, false);
			break;

		case CreateNotify :
			win_x = event.xcreatewindow.x;
			win_y = event.xcreatewindow.y;
			break;

		case ConfigureNotify :
			win_x = event.xconfigure.x;
			win_y = event.xconfigure.y;
			break;
		}
	}

}

void signal_handler(int sig)
{
	printf("Received signal %d, exiting...\n", sig);
	Sys_Quit();
	exit(0);
}

void InitSig(void)
{
	signal(SIGHUP, signal_handler);
	signal(SIGINT, signal_handler);
	signal(SIGQUIT, signal_handler);
	signal(SIGILL, signal_handler);
	signal(SIGTRAP, signal_handler);
	signal(SIGIOT, signal_handler);
	signal(SIGBUS, signal_handler);
	signal(SIGFPE, signal_handler);
	signal(SIGSEGV, signal_handler);
	signal(SIGTERM, signal_handler);
}

/*
=================
GL_BeginRendering

=================
*/
void GL_BeginRendering (int *x, int *y, int *width, int *height)
{
	*x = *y = 0;
	*width = scr_width;
	*height = scr_height;

    if (glXMakeCurrent( dpy, win, ctx ))
		Sys_Error ("glXMakeCurrent failed");

	glViewport (*x, *y, *width, *height);
}


void GL_EndRendering (void)
{
	glFlush();
	glXSwapBuffers(dpy, win);
}

void Sys_SendKeyEvents(void)
{
	HandleEvents();
}

/*
===============
CheckExtensions
===============
*/
void CheckExtensions (void) 
{
	const char	*gl_extensions;
	
	void *prjobj;
	
	MR1glMTexCoord2f=NULL;
	MR1glSelectTexture=NULL;
	MR1glLockArraysEXT=NULL;
	MR1glUnlockArraysEXT=NULL;
	
	
	gl_extensions = glGetString (GL_EXTENSIONS);

	Con_Printf ("GL_VENDOR: %s\n", glGetString (GL_VENDOR));
	Con_Printf ("GL_RENDERER: %s\n", glGetString (GL_RENDERER));

	// Check for Multitexture Extensions

	if ((prjobj = dlopen(NULL, RTLD_LAZY)) == NULL) {
		Con_Printf("Unable to open symbol list for main program.\n");
		return;
	}

	if (COM_CheckParm("-nomtex")) 
	{
		Con_Printf ("Multitexture Disabled\n");
	}
	else if (strstr (gl_extensions, "GL_ARB_multitexture")) 
	{
		Con_Printf ("Found GL_ARB_multitexture\n");
		MR1glMTexCoord2f	= (void *) dlsym(prjobj, "glMultiTexCoord2fARB");
		MR1glSelectTexture	= (void *) dlsym(prjobj,"glActiveTextureARB");
		TEXTURE0		= TEXTURE0_ARB;
		TEXTURE1		= TEXTURE1_ARB;
	} 
	else if (strstr (gl_extensions, "GL_SGIS_multitexture")) 
	{
		Con_Printf ("Found GL_SGIS_multitexture\n");
		MR1glMTexCoord2f	= (void *) dlsym(prjobj,"glMTexCoord2fSGIS");
		MR1glSelectTexture	= (void *) dlsym(prjobj,"glSelectTextureSGIS");
		TEXTURE0		= TEXTURE0_SGIS;
		TEXTURE1		= TEXTURE1_SGIS;
	}
	else
	{
		Con_Printf ("Multitexture Not Found\n");
	}

	if (MR1glMTexCoord2f && MR1glSelectTexture) {
		Con_Printf("Multitexture extensions found.\n");
		gl_mtexable = true;
	} else
		Con_Printf("Symbol not found, disabled.\n");
	
	dlclose(prjobj);

	// Check for Vertex Array Extensions

	if (COM_CheckParm("-nocva")) 
	{
		Con_Printf ("Compiled Vertex Arrays Disabled\n");
	}
	else if (strstr (gl_extensions, "GL_EXT_compiled_vertex_array"))
	{
		Con_Printf ("Found GL_EXT_compiled_vertex_array\n");
		MR1glLockArraysEXT		= (void *) dlsym (prjobj, "glLockArraysEXT");
		MR1glUnlockArraysEXT	= (void *) dlsym (prjobj, "glUnlockArraysEXT");
	}
	else
	{
		Con_Printf ("Compiled Vertex Arrays not found\n");
	}
	
	if (MR1glLockArraysEXT && MR1glUnlockArraysEXT) {
		Con_Printf("Vertex array  extensions found.\n");
		gl_mvertex = true;
	} else
		Con_Printf("Symbol not found, disabled.\n");

}

/*
=======
GL_Init
======
*/
void GL_Init (void)
{
	CheckExtensions();
	//CheckMultiTextureExtensions ();

	glEnable		(GL_TEXTURE_2D);
	glEnable		(GL_DITHER);
	glDisable		(GL_BLEND);
	glDisable		(GL_ALPHA_TEST);

	glHint			(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glCullFace		(GL_FRONT);
	glClearColor	(0,0,0,0);

	glBlendFunc		(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDepthFunc		(GL_LEQUAL);
	glDepthMask		(true);
	glDepthRange	(0,1);

	glLineWidth		(3);
	glFrustum		(-4, 4, -3, 3, 4, 4096);

	glTexEnvi		(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glShadeModel	(GL_SMOOTH);
	glPolygonMode	(GL_FRONT_AND_BACK, GL_FILL);

	glClear			(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

/*
============
VID_SetGamma
============
*/
int VID_SetGamma (float gamma, float scale, float base)
{
#ifdef VIDMODEXT_GAMMA_RAMP
	int i=0;
	
	BuildGammaTable (gamma, scale, base, &curgamma[0][0]);

	for (i = 0;i < 256;i++)
		curgamma[1][i] = curgamma[2][i] = curgamma[0][i];

	i = XF86VidModeSetGammaRamp (dpy, scrnum, 256,
		&curgamma[0][0], &curgamma[1][0], &curgamma[2][0]);
	return i;
#else
	return 0;
#endif
}

/*
==================
VID_GetSystemGamma
==================
*/
void VID_GetSystemGamma (void)
{
#ifdef VIDMODEXT_GAMMA_RAMP
	XF86VidModeGetGammaRamp (dpy, scrnum, 256,
		&sysgamma[0][0], &sysgamma[1][0], &sysgamma[2][0]);
#endif
}

/*
======================
VID_RestoreSystemGamma
======================
*/
void VID_RestoreSystemGamma (void)
{
#ifdef VIDMODEXT_GAMMA_RAMP
	XF86VidModeSetGammaRamp (dpy, scrnum, 256,
		&sysgamma[0][0], &sysgamma[1][0], &sysgamma[2][0]);
#endif
}

/*
============
VID_Shutdown
============
*/
void VID_Shutdown (void)
{
	if (!ctx || !dpy)
		return;
	uninstall_grabs();

	//restore original gamma settings	
	VID_RestoreSystemGamma ();
	    
	if (dpy) {
		if (ctx)
			glXDestroyContext(dpy, ctx);
		if (win)
			XDestroyWindow(dpy, win);
		if (vidmode_active)
			XF86VidModeSwitchToMode(dpy, scrnum, vidmodes[0]);
		XCloseDisplay(dpy);
	}
	vidmode_active = false;
	dpy = NULL;
	win = 0;
	ctx = NULL;
	
	//ChangeDisplaySettings (NULL, 0);
	
	//ShowCursor (true);	
}

/*
==============
ClearAllStates
==============
*/
void ClearAllStates (void)
{
	int		i;
	
	for (i=0 ; i<256 ; i++)
		Key_Event (i, false);

	Key_ClearStates ();
}

/*
========
VID_Init
========
*/
void VID_Init (void)
{
	/* we want to use GLX, functions taken from WinQuake/vid_linuxglx.c
	 * and rewritten from GL tutorials
	 */
	 
	int		i;
	int attrib[] = {
	GLX_RGBA,
	GLX_RED_SIZE, 1,
	GLX_GREEN_SIZE, 1,
	GLX_BLUE_SIZE, 1,
	GLX_DOUBLEBUFFER,
	GLX_DEPTH_SIZE, 1,
	None
	};
	XSetWindowAttributes attr;
	unsigned long mask;
	Window root;
	int MajorVersion, MinorVersion;
	int actualWidth, actualHeight;

	
	Cvar_RegisterVariable (&vid_mode);

	mode.width		= 640;
	mode.height		= 480;
	mode.fullscreen	= true;

// interpret command-line params

// set vid parameters	(added mode. statements for MiniRacer)
	i = COM_CheckParm ("-width");
	if (i)
		mode.width		= MR_atoi (com_argv[i+1]);

	i = COM_CheckParm ("-height");
	if (i)
		mode.height		= MR_atoi (com_argv[i+1]);

	i = COM_CheckParm ("-window");
	if (i)
		mode.fullscreen	= false;

	vid.width			= mode.width;
	vid.height			= mode.height;

	// open display and create main window

	if (!(dpy = XOpenDisplay(NULL))) {
	    Sys_Error("Error: couldn't open the X display\n");
	    exit(1);
	}

	scrnum = DefaultScreen(dpy);
	root = RootWindow(dpy, scrnum);
	
	// Get video mode list
	if (!XF86VidModeQueryVersion(dpy, &MajorVersion, &MinorVersion)) { 
		vidmode_ext = false;
	} else {
		Con_Printf("Using XFree86-VidModeExtension Version %d.%d\n", MajorVersion, MinorVersion);
		vidmode_ext = true;
	}

	visinfo = glXChooseVisual(dpy, scrnum, attrib);
	if (!visinfo) {
		fprintf(stderr, "qkHack: Error couldn't get an RGB, Double-buffered, Depth visual\n");
		exit(1);
	}

	if (vidmode_ext) {
		int best_fit, best_dist, dist, x, y;
		
		XF86VidModeGetAllModeLines(dpy, scrnum, &num_vidmodes, &vidmodes);

		// get system gamma and update gamma correcture
		VID_GetSystemGamma ();
		VID_UpdateGamma (true);

		// Are we going fullscreen?  If so, let's change video mode
		if (mode.fullscreen) {
			best_dist = 9999999;
			best_fit = -1;

			for (i = 0; i < num_vidmodes; i++) {
				if (mode.width > vidmodes[i]->hdisplay ||
				    mode.height > vidmodes[i]->vdisplay)
					continue;

				x = mode.width - vidmodes[i]->hdisplay;
				y = mode.height - vidmodes[i]->vdisplay;
				dist = (x * x) + (y * y);
				if (dist < best_dist) {
					best_dist = dist;
					best_fit = i;
				}
			}

			if (best_fit != -1) {
				actualWidth = vidmodes[best_fit]->hdisplay;
				actualHeight = vidmodes[best_fit]->vdisplay;

				// change to the mode
				XF86VidModeSwitchToMode(dpy, scrnum, vidmodes[best_fit]);
				vidmode_active = true;

				// Move the viewport to top left
				XF86VidModeSetViewPort(dpy, scrnum, 0, 0);
			} else
				mode.fullscreen = 0;
		}
	}


	// window attributes	
	attr.background_pixel = 0;
	attr.border_pixel = 0;
	attr.colormap = XCreateColormap(dpy, root, visinfo->visual, AllocNone);
	attr.event_mask = X_MASK;

	if (vidmode_active) {
		mask = CWBackPixel | CWColormap | CWSaveUnder | CWBackingStore | 
			CWEventMask | CWOverrideRedirect;
		attr.override_redirect = True;
		attr.backing_store = NotUseful;
		attr.save_under = False;
	} else
		mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

	win = XCreateWindow(dpy, root, 0, 0, vid.width, vid.height,
						0, visinfo->depth, InputOutput,
						visinfo->visual, mask, &attr);
	
	XStoreName(dpy, win, "MiniRacer");

	XMapWindow(dpy, win);
	
	if (vidmode_active) {
		XMoveWindow(dpy, win, 0, 0);
		XRaiseWindow(dpy, win);
		XWarpPointer(dpy, None, win, 0, 0, 0, 0, 0, 0);
		XFlush(dpy);
		// Move the viewport to top left
		XF86VidModeSetViewPort(dpy, scrnum, 0, 0);
	}
	
	// now grabbing the mouse cursor
	install_grabs();

	XFlush(dpy);

	// Settings for the OpenGL window

	ctx = glXCreateContext(dpy, visinfo, NULL, True);

	glXMakeCurrent(dpy, win, ctx);	
	
	ClearAllStates ();

	InitSig(); // trap evil signals
	
	GL_Init ();
	
	Con_SafePrintf ("Video mode %dx%d initialized.\n", mode.width, mode.height);

}

// wrapper functions: avoid segfault when multitexture extensions are not present
 
void MRglMTexCoord2f	(GLenum target , GLfloat v1, GLfloat v2)
{
    if(!gl_mtexable)
	return;
    MR1glMTexCoord2f	(target, v1, v2);
}
void MRglSelectTexture	(GLenum target)
{
    if(!gl_mtexable)
	return;
    MR1glSelectTexture	(target);
}
void MRglLockArraysEXT(int first, int count)
{
    if(!gl_mvertex)
	return;
    MR1glLockArraysEXT(first, count);
}
void MRglUnlockArraysEXT(void)
{
    if(!gl_mvertex)
	return;
    MR1glUnlockArraysEXT();
}

// wrappers for win32 mouse handler functions
void GetCursorPos(POINT *pointer_pos)
{
    int dummy;
    Window wdummy;
    
    XQueryPointer(dpy, win, &wdummy, &wdummy, &dummy, &dummy,
		  &pointer_pos->x, &pointer_pos->y, &dummy);
}

void SetCursorPos(int x, int y)
{
    XWarpPointer(dpy, None, win, 0, 0, 0, 0, x, y);
}

