/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __PROGS__
#define __PROGS__

#define		FOFS(x) (((int)&(((entvars_t *)0)->x) / sizeof(float)))
#define		MAX_ENT_LEAFS	16
#define		ENTVAR_COUNT 10

typedef enum
{
	ev_void,
	ev_string, 
	ev_float, 
	ev_vector, 
	ev_entity, 
	ev_field, 
	ev_function, 
	ev_pointer
} etype_t;

typedef struct
{
	unsigned short	type;
	unsigned short	ofs;
	char			*s_name;
} ddef_t;

typedef struct
{
	vec3_t		absmin;
	vec3_t		absmax;
	vec3_t		angles;
	vec3_t		origin;

	float		number;
	float		modelindex;

	char		*wad;
	char		*model;
	char		*message;
	char		*classname;
} entvars_t;

typedef struct
{
	float		way_timer;
	float		nextthink;
	float		mins[3];
	float		maxs[3];
	float		size[3];
	float		origin[3];
	float		laptime[5];
	float		velocity[3];
	float		goalorigin[3];

	int			bot_speed;
	int			yaw_speed;
	int			ideal_yaw;

	byte		ways;
	byte		laps;	
	byte		back;
	byte		left;
	byte		right;
	byte		brake;
	byte		turbo;
	byte		solid;
	byte		flags;
	byte		forward;
	byte		impulse;
	byte		ClientNo;
	byte		movetype;
	byte		laptrigger;
	byte		color[3];
	
	char		*classname;
	char		netname[32];

	qboolean	fly;
	qboolean	ready;
	qboolean	isbot;
	qboolean	Active;
	qboolean	finished;
	qboolean	havegoal;

	void		(*touch)(void);
	void		(*think)(void);
} game_t;

typedef struct edict_s
{
	game_t			game;
	entvars_t		v;
	link_t			area;
	entity_state_t	baseline;
	float			freetime;
	int				num_leafs;
	short			leafnums[MAX_ENT_LEAFS];
	qboolean		free;
} edict_t;

typedef struct point_s
{
	float		origin[3];
	float		angles[3];
	qboolean	used;
} point_t;

typedef struct miniracer_s
{
	edict_t	*self;
	edict_t	*other;
	edict_t	*world;

	point_t	waypoint[64];
	point_t	startpoint[8];
	point_t	finishpoint[8];
	
	float	tele_timer;

	byte	TotalWays;
	byte	MaxClients;
	byte	FinishPlace;
	byte	started;
} miniracer_t;

#define	EDICT_FROM_AREA(l) STRUCT_FROM_LINK(l,edict_t,area)

//============================================================================

extern	miniracer_t		miniracer;

//============================================================================


int		NUM_FOR_EDICT(edict_t *e);

char	*ED_NewString (char *string);
char	*ED_ParseEdict (char *data, edict_t *ent);

void	ED_Free (edict_t *ed);
void	ED_LoadFromFile (char *data);

edict_t	*ED_Alloc (void);
edict_t	*EDICT_NUM (int n);

#define	NEXT_EDICT(e) ((edict_t *)( (byte *)e + sizeof(edict_t)))
#define	EDICT_TO_PROG(e) ((byte *)e - (byte *)sv.edicts)
#define PROG_TO_EDICT(e) ((edict_t *)((byte *)sv.edicts + e))

#endif //__PROGS__
