/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"
#include	"../sound.h"

#define FixVolume(v) ((v) = ((v) * 500) - 5000)

extern	camera_t	camera;
extern	qboolean	started_engine;
extern	qboolean	scr_drawmenu;
extern	qboolean	scr_drawconsole;

cvar_t		track = {"track", "0", true};

int			wavnum_adder;
int			music_volume;

qboolean	engine_started	= false;
qboolean	music_playing	= false;

void S_Init (void)
{
	engine_started	= false;
	music_playing	= false;

#ifdef SOUND
	Cvar_RegisterVariable (&track);

	if (!MRS_Init ())
	{
		Con_Printf("Sound Initialization failed.\n\n");
		return;
	}

	Con_Printf("Sound Initialization successful.\n\n");

	MRS_SampleLoad (0, "sounds/misc/menu1.wav");
	MRS_SampleLoad (1, "sounds/misc/menu2.wav");
	MRS_SampleLoad (2, "sounds/misc/boop.wav");
	MRS_SampleLoad (3, "sounds/misc/beep.wav");

	MRS_SampleLoad (4, "sounds/misc/skid.wav");
	MRS_SampleLoad (5, "sounds/misc/skid.wav");
	MRS_SampleLoad (6, "sounds/misc/skid.wav");
	MRS_SampleLoad (7, "sounds/engine/f40.wav");
	MRS_SampleLoad (8, "sounds/music/menu1.wav");
	MRS_SampleLoad (9, "sounds/music/menu2.wav");
	MRS_SampleLoad (10,"sounds/music/menu3.wav");
	MRS_SampleLoad (11,"sounds/music/menu4.wav");

	Con_Printf("Sound loading successful.\n\n");
#else
	Con_Printf("Sound engine not available.\n\n");
#endif
}

void S_Shutdown (void)
{
#ifdef SOUND
	MRS_SampleFree (0);
	MRS_SampleFree (1);
	MRS_SampleFree (2);
	MRS_SampleFree (3);
	MRS_SampleFree (4);
	MRS_SampleFree (5);
	MRS_SampleFree (6);
	MRS_SampleFree (7);
	MRS_SampleFree (8);
	MRS_SampleFree (9);
	MRS_SampleFree (10);
	MRS_SampleFree (11);

	MRS_Shutdown ();
#endif
}

void S_Stop_Music (void)
{
#ifdef SOUND
	MRS_SampleStop (8);
	MRS_SampleStop (9);
	MRS_SampleStop (10);
	MRS_SampleStop (11);
#endif

	music_playing = false;
}

void S_Play_Music (int volume)
{
#ifdef SOUND
	int		wav;

	FixVolume (volume);

	if (volume != music_volume)
	{
		MRS_SampleSetVolume (8, volume);
		MRS_SampleSetVolume (9, volume);
		MRS_SampleSetVolume (10, volume);
		MRS_SampleSetVolume (11, volume);
		music_volume = volume;
	}
#endif

	if (music_playing)
		return;

#ifdef SOUND
	if (track.ivalue < 3)
		track.ivalue++;
	else
		track.ivalue = 0;

	Cvar_SetValue ("track", track.ivalue);
	
	wav = track.ivalue + 8;

	MRS_SamplePlay (wav, volume, true);
#endif

	music_playing = true;
}

void S_Stop_Sound (int wavnum)
{
#ifdef SOUND
	if (wavnum == 7)
	{
		MRS_SampleSetPitch (7, 11025);
		engine_started = false;
	}

	MRS_SampleStop (wavnum);
#endif
}

void S_Play_2DSound (byte wavnum, int volume)
{
#ifdef SOUND
	FixVolume (volume);

	MRS_SamplePlay (wavnum, volume, false);
#endif
}

void S_Play_3DSound (byte wavnum, int volume, vec3_t origin)
{
#ifdef SOUND
	if (wavnum == 4)
	{
		wavnum += wavnum_adder;
		++wavnum_adder;

		if (wavnum_adder == 3)
			wavnum_adder = 0;
	}

	FixVolume (volume);

	if(!scr_drawconsole && !scr_drawmenu)
		MRS_SamplePlay (wavnum, volume, false);
#endif
}

void S_Play_EngineSound (byte wavnum, int volume, vec3_t origin, float speed)
{
#ifdef SOUND
	if (!scr_drawconsole && !scr_drawmenu && !engine_started)
	{
		engine_started = true;

		FixVolume (volume);
		
		MRS_SamplePlay (wavnum, volume, true);
	}

	MRS_SampleSetPitch (wavnum, 15000 * (0.5f + speed));
#endif
}
